import sys
import platform
import os
import json
import logging
import requests

def apply_policy(policy_names, asset_id_list, args):
    url = "https://" + args.instance + "/api/v1/policies/apply/"
    auth_data = "?handle=" + args.handle + "&token=" + args.token + "&format=json"
    policy_names_list = policy_names.split(',')
    payload = { "asset_ids": asset_id_list, "policy_names": policy_names_list }
    resp = requests.post(url + auth_data, json=payload)
    if resp.status_code == 200:
        logging.info("Applying specified policy....")
        policy_job_id = resp.json()['policy_job_id']
        logging.info("Policy job started...job id is [%s]. This may take some time...", policy_job_id)
        return policy_job_id
    else:
        logging.error("Error applying specified policy")
        logging.error("Response details: %s", resp.content)
        sys.exit(1)

def is_policy_job_done(policy_job_id, args):
    url = "https://" + args.instance + "/api/v1/policyjobs/" + policy_job_id + "/"
    auth_data = "?handle=" + args.handle + "&token=" + args.token + "&format=json"
    resp = requests.get(url + auth_data)
    if resp.status_code == 200:
        policy_job_json = resp.json()
        if policy_job_json['status'] == "COMPLETED":
            return True, policy_job_json
        else:
            return False, policy_job_json
    else:
        logging.error("Error retrieving policy job details for [%s]", policy_job_id)
        logging.error("Response details: %s", resp.content)
        return False, None

def process_policy_job_actions(pj_json):
    exit_with_code = False
    exit_code = None
    final_actions = { }
    policies = pj_json['policy_json']
    policies_outcome = pj_json['policy_outcome']
    for policy in policies:
        pn = policy['name']
        pa = policy['actions']
        policy_outcome = policies_outcome[pn]
        for assetid in policy_outcome.keys():
            po = policy_outcome[assetid]
            if po == "PASSED":
                lookup = 'on_pass'
            elif po == "FAILED":
                lookup = 'on_fail'
            if final_actions.get(pn) is None:
                final_actions[pn] = { }
            if pa.get(lookup) is not None:
                final_actions[pn][assetid] = pa[lookup]
                if "exit_with_code" in pa[lookup].keys():
                    exit_with_code = True
                    exit_code = pa[lookup]['exit_with_code']

    # Process policies with actions other than 'exit_with_code' in final_actions
    # TODO later on

    logging.info("Policy evaluation is done...")
    if exit_with_code:
        return exit_code
    else:
        return None




import sys
import subprocess
import os

from . import check1 as check1
from . import check2 as check2
from . import check3 as check3
from . import check4 as check4
from . import check5 as check5
from . import check6 as check6
from . import check7 as check7
from . import gcp_cis_utils as gcp_cis_utils

def run_tests(args):
    gcp_cis_utils.set_encoding(args.encoding)
    config_issues = []
    config_issues.extend(check1.run_checks())
    config_issues.extend(check2.run_checks())
    config_issues.extend(check3.run_checks())
    config_issues.extend(check4.run_checks())
    config_issues.extend(check5.run_checks())
    config_issues.extend(check6.run_checks())
    #config_issues.extend(check7.run_checks())
    #print config_issues
    return config_issues

# run_tests()



import os
import json
import logging
import requests
from . import az_cis_utils as az_cis_utils

def check21():
    logging.info("Processing 21...")
    return "Check not available with azure CLI"

def check22(subid):
    logging.info("Processing 22...")
    try:
        query20=('az account get-access-token --subscription %s --query [accessToken]' % subid)
        score22=['<font color="red">Failed</font>',0]
        score23=['<font color="red">Failed</font>',0]
        score24=['<font color="red">Failed</font>',0]
        score25=['<font color="red">Failed</font>',0]
        score26=['<font color="red">Failed</font>',0]
        score27=['<font color="red">Failed</font>',0]
        score28=['<font color="red">Failed</font>',0]
        score29=['<font color="red">Failed</font>',0]
        score210=['<font color="red">Failed</font>',0]
        score211=['<font color="red">Failed</font>',0]
        score212=['<font color="red">Failed</font>',0]
        score213=['<font color="red">Failed</font>',0]
        score214=['<font color="red">Failed</font>',0]
        score215=['<font color="red">Failed</font>',0]
        score216=['<font color="red">Failed</font>',0]
        score217=['<font color="red">Failed</font>',0]
        score218=['<font color="red">Failed</font>',0]
        score219=['<font color="red">Failed</font>',0]
        json_cis20=az_cis_utils.query_az(query20)
        access_token=json_cis20[0]
        headers = {"Authorization": 'Bearer ' + access_token}
        request = ('https://management.azure.com/subscriptions/%s/providers/microsoft.Security/policies?api-version=2015-06-01-preview' % subid)
        try:
            json_output = requests.get(request, headers=headers).json()
            value22=json_output['value'][0]['properties']['logCollection']
            value23=json_output['value'][0]['properties']['recommendations']['patch']
            value24=json_output['value'][0]['properties']['recommendations']['baseline']
            value25=json_output['value'][0]['properties']['recommendations']['antimalware']
            value26=json_output['value'][0]['properties']['recommendations']['diskEncryption']
            value27=json_output['value'][0]['properties']['recommendations']['nsgs']
            value28=json_output['value'][0]['properties']['recommendations']['waf']
            value29=json_output['value'][0]['properties']['recommendations']['ngfw']
            value210=json_output['value'][0]['properties']['recommendations']['vulnerabilityAssessment']
            value211=json_output['value'][0]['properties']['recommendations']['storageEncryption']
            value212=json_output['value'][0]['properties']['recommendations']['jitNetworkAccess']
            value213=json_output['value'][0]['properties']['recommendations']['appWhitelisting']
            value214=json_output['value'][0]['properties']['recommendations']['sqlAuditing']
            value215=json_output['value'][0]['properties']['recommendations']['sqlTde']
            value216=json_output['value'][0]['properties']['securityContactConfiguration']['securityContactEmails']
            value217=json_output['value'][0]['properties']['securityContactConfiguration']['securityContactPhone']
            value218=json_output['value'][0]['properties']['securityContactConfiguration']['areNotificationsOn']
            value219=json_output['value'][0]['properties']['securityContactConfiguration']['sendToAdminOn']
            if (value22=="On"):
                score22=['<font color="green">Passed</font>',1]
            if (value23=="On"):
                score23=['<font color="green">Passed</font>',1]
            if (value24=="On"):
                score24=['<font color="green">Passed</font>',1]
            if (value25=="On"):
                score25=['<font color="green">Passed</font>',1]
            if (value26=="On"):
                score26=['<font color="green">Passed</font>',1]
            if (value27=="On"):
                score27=['<font color="green">Passed</font>',1]
            if (value28=="On"):
                score28=['<font color="green">Passed</font>',1]
            if (value29=="On"):
                score29=['<font color="green">Passed</font>',1]
            if (value210=="On"):
                score210=['<font color="green">Passed</font>',1]
            if (value211=="On"):
                score211=['<font color="green">Passed</font>',1]
            if (value212=="On"):
                score212=['<font color="green">Passed</font>',1]
            if (value213=="On"):
                score213=['<font color="green">Passed</font>',1]
            if (value214=="On"):
                score214=['<font color="green">Passed</font>',1]
            if (value215=="On"):
                score215=['<font color="green">Passed</font>',1]
            if (value216!="[]"):
                score216=['<font color="green">Passed</font>',1]
            if (value217!=""):
                score217=['<font color="green">Passed</font>',1]
            if (value218):
                score218=['<font color="green">Passed</font>',1]
            if (value219):
                score219=['<font color="green">Passed</font>',1]                                                                        
            chk22=('Current Setting: <font color="blue"> %s</b></font>' % value22)
            chk23=('Current Setting: <font color="blue"> %s</b></font>' % value23)
            chk24=('Current Setting: <font color="blue"> %s</b></font>' % value24)
            chk25=('Current Setting: <font color="blue"> %s</b></font>' % value25)
            chk26=('Current Setting: <font color="blue"> %s</b></font>' % value26)
            chk27=('Current Setting: <font color="blue"> %s</b></font>' % value27)
            chk28=('Current Setting: <font color="blue"> %s</b></font>' % value28)
            chk29=('Current Setting: <font color="blue"> %s</b></font>' % value29)
            chk210=('Current Setting: <font color="blue"> %s</b></font>' % value210)
            chk211=('Current Setting: <font color="blue"> %s</b></font>' % value211)
            chk212=('Current Setting: <font color="blue"> %s</b></font>' % value212)
            chk213=('Current Setting: <font color="blue"> %s</b></font>' % value213)
            chk214=('Current Setting: <font color="blue"> %s</b></font>' % value214)
            chk215=('Current Setting: <font color="blue"> %s</b></font>' % value215)
            chk216=('Current Setting: <font color="blue"> %s</b></font>' % value216)
            chk217=('Current Setting: <font color="blue"> %s</b></font>' % value217)
            chk218=('Current Setting: <font color="blue"> %s</b></font>' % value218)
            chk219=('Current Setting: <font color="blue"> %s</b></font>' % value219)
            return [chk22,chk23,chk24,chk25,chk26,chk27,chk28,chk29,chk210,chk211,chk212,chk213,chk214,chk215,chk216,chk217,chk218,chk219,score22,score23,score24,score25,score26,score27,score28,score29,score210,score211,score212,score213,score214,score215,score216,score217,score218,score219]
        except Exception as e:
            logging.error("Exception in check2: %s %s" %(type(e), str(e.args)))
            unkScore=['<font color="orange">UNKNOWN </font>',0]
            chk="Failed to make API call"
            return [chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore]
    except Exception as e:
        logging.error("Exception in check2: %s %s" %(type(e), str(e.args)))
        unkScore=['<font color="orange">UNKNOWN </font>',0]
        chk="Failed to Query"
        return [chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,chk,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore]




import os
import json
import logging
import sys
from . import az_cis_utils as az_cis_utils

def check62(subid):
    logging.info("Processing 61 and 62...")
    acl61=""
    acl62=""
    failvalue61 = 0
    passvalue61 = 0
    totalvalue61 = 0
    nsgfailvalue61 = 0
    score61=""
    passed61='<font color="green">Passed </font>'
    failvalue62 = 0
    passvalue62 = 0
    totalvalue62 = 0
    nsgfailvalue62 = 0
    score62=""
    passed62='<font color="green">Passed </font>'
    rangestart=1
    rangeend=65535
    rangestarts=1
    rangeends=65535
    try:
        query62='az network nsg list --query [*].[name,securityRules,id,resourceGroup]'
        #query62=('az network nsg list --query "[?contains(id,\'%s\')].[name,securityRules]"' % subid)
        json_cis=az_cis_utils.query_az(query62)
        #i iteration number of NSG
        if (len(json_cis)>0):
            for i in range(len(json_cis)):
                nsgid=str(json_cis[i][2])
                nsgrd=str(json_cis[i][3])
                # Find NSG with public IP @
                query621=('az network nic list -g %s --query "[?networkSecurityGroup.id==\'%s\'].ipConfigurations[0].publicIpAddress"'  % (nsgrd,nsgid))
                json_cis2=az_cis_utils.query_az(query621)
                #Check if public IP address is not empty
                if (len(json_cis2)>0):
                    #j iteration of ACL per NSG
                    if (len(json_cis[i][1])>0):
                        for j in range(len(json_cis[i][1])):
                            protocol=str(json_cis[i][1][j]['protocol'])
                            dport=str(json_cis[i][1][j]['destinationPortRange'])
                            dports=json_cis[i][1][j]['destinationPortRanges']
                            action=str(json_cis[i][1][j]['access'])
                            src=str(json_cis[i][1][j]['sourceAddressPrefix'])
                            direction=str(json_cis[i][1][j]['direction'])
                            # Combination of ranges or single port
                            # Split in case a single range is used
                            if (dport!="None" and dport!="*"):
                                rangedport=dport.split('-')
                                rangestart=int(rangedport[0])
                                # Check if single range configured
                                if (len(rangedport)>1):
                                    rangeend=int(rangedport[1])
                                    ## Check For Inbound RDP Access
                                    ## Available Protocol TCP, UDP or * or Internet (Service Tag)
                                    if (protocol!="UDP" and (rangestart<=3389<=rangeend) and action=="Allow" and (src=="*" or src=="Internet") and direction=="Inbound"):
                                        acl61=acl61+('Inbound RDP Allowed on nsg <b>%s</b><br>\n' % (str(json_cis[i][0])))
                                        passed61='<font color="red">Failed </font>'
                                        failvalue61=1
                                    ## Check For Inbound SSH Access.
                                    if (protocol !="UDP" and (rangestart<=22<=rangeend) and action=="Allow" and (src=="*" or src=="Internet") and direction=="Inbound"):
                                        acl62=acl62+('Inbound SSH Allowed on nsg <b>%s</b><br>\n' % (str(json_cis[i][0])))
                                        passed62='<font color="red">Failed </font>'
                                        failvalue62=1
                                else:
                                    if (protocol!="UDP" and (rangestart==3389) and action=="Allow" and (src=="*" or src=="Internet") and direction=="Inbound"):
                                        acl61=acl61+('Inbound RDP Allowed on nsg <b>%s</b><br>\n' % (str(json_cis[i][0])))
                                        passed61='<font color="red">Failed </font>'
                                        failvalue61=1
                                    ## Check For Inbound SSH Access.
                                    if (protocol !="UDP" and (rangestart==22) and action=="Allow" and (src=="*" or src=="Internet") and direction=="Inbound"):
                                        acl62=acl62+('Inbound SSH Allowed on nsg <b>%s</b><br>\n' % (str(json_cis[i][0])))
                                        passed62='<font color="red">Failed </font>'
                                        failvalue62=1

                            # Combination of ranges and single port
                            if (len(dports)>0):    
                                for k in range(len(dports)):
                                    rangedports=dports[k].split('-')
                                    rangestarts=int(rangedports[0])             
                                    if (len(rangedports)>1):
                                        rangeends=int(rangedports[1])
                                        ## Check For Inbound RDP Access
                                        ## Available Protocol TCP, UDP or * or Internet (Service Tag)
                                        if (protocol!="UDP" and (rangestarts<=3389<=rangeends) and action=="Allow" and (src=="*" or src=="Internet") and direction=="Inbound"):
                                            acl61=acl61+('Inbound RDP Allowed on nsg <b>%s</b><br>\n' % (str(json_cis[i][0])))
                                            passed61='<font color="red">Failed </font>'
                                            failvalue61=1
                                        ## Check For Inbound SSH Access.
                                        if (protocol !="UDP" and (rangestarts<=22<=rangeends) and action=="Allow" and (src=="*" or src=="Internet") and direction=="Inbound"):
                                            acl62=acl62+('Inbound SSH Allowed on nsg <b>%s</b><br>\n' % (str(json_cis[i][0])))
                                            passed62='<font color="red">Failed </font>'
                                            failvalue62=1
                                    else:
                                        if (protocol!="UDP" and (rangestarts==3389) and action=="Allow" and (src=="*" or src=="Internet") and direction=="Inbound"):
                                            acl61=acl61+('Inbound RDP Allowed on nsg <b>%s</b><br>\n' % (str(json_cis[i][0])))
                                            passed61='<font color="red">Failed </font>'
                                            failvalue61=1
                                        ## Check For Inbound SSH Access.
                                        if (protocol !="UDP" and (rangestarts==22) and action=="Allow" and (src=="*" or src=="Internet") and direction=="Inbound"):
                                            acl62=acl62+('Inbound SSH Allowed on nsg <b>%s</b><br>\n' % (str(json_cis[i][0])))
                                            passed62='<font color="red">Failed </font>'
                                            failvalue62=1

                            ## Check if all port are opened
                            if ((dport=="*") and action=="Allow" and (src=="*" or src=="Internet") and direction=="Inbound"):
                                acl61=acl61+('<font color="red">All Inbound ports are opened on nsg <b>%s</b></font><br>\n' % (str(json_cis[i][0])))
                                acl62=acl62+('<font color="red">All Inbound ports are opened on nsg <b>%s</b></font><br>\n' % (str(json_cis[i][0])))
                                passed61='<font color="red">Failed </font>'
                                passed62='<font color="red">Failed </font>'
                                failvalue61=1
                                failvalue62=1
                        if (failvalue61==1):
                            # Incread counter for NSG which is not compliant
                            nsgfailvalue61=nsgfailvalue61+1
                            #Reset counter for Next NSG
                            failvalue61=0 
                        else:
                            acl61=acl61+('No Inbound RDP Allowed on nsg <b>%s</b><br>\n' % (str(json_cis[i][0])))
                        if(failvalue62==1):
                            # Incread counter for NSG which is not compliant
                            nsgfailvalue62=nsgfailvalue62+1
                            #Reset counter for Next NSG
                            failvalue62=0
                        else:
                            acl62=acl62+('No Inbound SSH Allowed on nsg <b>%s</b><br>\n' % (str(json_cis[i][0])))
                    # If No ACL defined for NSG, assumed RDP/SSH not allowed
                    else:
                        acl61=acl61+('No ACL defined on nsg <b>%s</b><br>\n' % (str(json_cis[i][0])))
                        acl62=acl62+('No ACL defined on nsg <b>%s</b><br>\n' % (str(json_cis[i][0])))
                else:
                    acl61=acl61+('No public interface defined on nsg <b>%s</b><br>\n' % (str(json_cis[i][0])))
                    acl62=acl62+('No public interface defined on nsg <b>%s</b><br>\n' % (str(json_cis[i][0])))
                #Increasing counter of NSG
                totalvalue61 = totalvalue61+1
                totalvalue62 = totalvalue62+1                
                passvalue61=totalvalue61-nsgfailvalue61
                passvalue62=totalvalue62-nsgfailvalue62
        else:
            acl61="No NSG Configured"
            acl62="No NSG Configured"
            totalvalue61 = 1
            totalvalue62 = 1
            passvalue61 = 1
            passvalue62 = 1
        score61=[acl61,passvalue61,totalvalue61,passed61]
        score62=[acl62,passvalue62,totalvalue62,passed62]
        return [score61,score62]
    except Exception as e:
        logging.error("Exception in check62: %s %s" %(type(e), str(e.args)))
        acl61="Failed to query for NSG or SSH/RDP not allowed"
        acl62="Failed to query for NSG or SSH/RDP not allowed"
        passed61='<font color="orange">UNKNOWN </font>'
        passed62='<font color="orange">UNKNOWN </font>'
        totalvalue61 = 1
        totalvalue62 = 1
        score61=[acl61,passvalue61,totalvalue61,passed61]
        score62=[acl62,passvalue62,totalvalue62,passed62]
        return [score61,score62]

def check63():
    logging.info("Processing 63...")
    st63=""
    passvalue63 = 0
    totalvalue63 = 0
    score63=""
    passed63='<font color="green">Passed </font>'
    try:
        query63='az sql server list --query [*][resourceGroup,name]'
        #query63=('az network nsg list --query "[?contains(id,\'%s\')].[resourceGroup,name]"' % subid)
        json_cis=az_cis_utils.query_az(query63)
        #iteration through SQL Servers
        if (len(json_cis)>0):
            for i in range(len(json_cis)):
                RG = json_cis[i][0]
                SRV = json_cis[i][1]
                queryrp=("az sql server firewall-rule list --resource-group %s --server %s --query [*].[startIpAddress,endIpAddress]" % (RG,SRV))
                try:
                    json_cis2=az_cis_utils.query_az(queryrp)
                    startip=str(json_cis2[0][0])
                    endip=str(json_cis2[0][1])
                    if (startip=="0.0.0.0" and endip=="0.0.0.0"):
                        passed63='<font color="red">Failed </font>'
                        st63=st63+('SQL Server: <b>%s</b> access is not restricted<br></li>\n' % SRV)  
                    else:
                        passvalue63=passvalue63+1
                        st63=st63+('SQL Server: <b>%s</b> access is restricted<br></li>\n' % SRV) 
                    totalvalue63 = totalvalue63+1
                except Exception as e:
                    logging.error('Failed to query SQL Server ' + str(e))
                    st63="Failed to query SQL Server"
                    passed63='<font color="orange">UNKNOWN </font>'
                    totalvalue63 = 1
                    score63=[st63,passvalue63,totalvalue63,passed63]
                    return score63
        else:
            st63="No SQL Server Configured"
            passvalue63 = 1
            totalvalue63 = 1
        score63=[st63,passvalue63,totalvalue63,passed63]
        return score63
    except Exception as e:
        logging.error("Exception in check63: %s %s" %(type(e), str(e.args)))
        st63="Failed to query SQL Server"
        totalvalue63 = 1
        passed63='<font color="orange">UNKNOWN </font>'
        score63=[st63,passvalue63,totalvalue63,passed63]
        return score63


def check64(subid):
    logging.info("Processing 64...")
    st64=""
    passvalue64 = 0
    totalvalue64 = 0
    score64=""
    passed64='<font color="green">Passed </font>'
    try:
        query64='az network nsg list --query [*][resourceGroup,name]'
        #query64=('az network nsg list --query "[?contains(id,\'%s\')].[resourceGroup,name]"' % subid)
        json_cis=az_cis_utils.query_az(query64)
        #iteration through NSG
        if (len(json_cis)>0):
            for i in range(len(json_cis)):
                RG = json_cis[i][0]
                NSG = json_cis[i][1]
                queryrp=("az network watcher flow-log show --resource-group %s --nsg %s" % (RG,NSG))
                try:
                    json_cis2=az_cis_utils.query_az(queryrp)
                    status=str(json_cis2['retentionPolicy']['enabled'])
                    days=json_cis2['retentionPolicy']['days']
                    if (days<90 or status is False):
                        passed64='<font color="red">Failed </font>'   
                    else:
                        passvalue64=passvalue64+1
                    totalvalue64 = totalvalue64+1
                    st64=st64+('NSG: <b>%s</b> Enabled: <font color="blue"><b>%s</b></font> Days <font color="blue"><b>%d</b></font><br></li>\n' % (NSG,status,days))
                except Exception as e:
                    logging.error('Failed to query for network watcher flow-log ' + str(e))
                    st64="Failed to query for network watcher flow-log"
                    passed64='<font color="orange">UNKNOWN </font>'
                    totalvalue64 = 1
                    score64=[st64,passvalue64,totalvalue64,passed64]
                    return score64
        else:
            st64="No NSG Configured"
            passvalue64 = 1
            totalvalue64 = 1
        score64=[st64,passvalue64,totalvalue64,passed64]
        return score64
    except Exception as e:
        logging.error("Exception in check64: %s %s" %(type(e), str(e.args)))
        st64="Failed to query for NSG"
        totalvalue64 = 1
        passed64='<font color="orange">UNKNOWN </font>'
        score64=[st64,passvalue64,totalvalue64,passed64]
        return score64


def check65(subid):
    logging.info("Processing 65...")
    passed65='<font color="red">Failed </font>'
    numberegions=0
    totalregions = 27
    try:
        query65='az network watcher list'
        #query65=('az network watcher list --query "[?contains(id,\'%s\')]"' % subid)
        json_cis=az_cis_utils.query_az(query65)
        if (len(json_cis)>0):
            #iteration through existing regions
            for i in range(len(json_cis)):
                numberegions = numberegions+1
                region = json_cis[i]['location']
                state = json_cis[i]['provisioningState']
            if (numberegions==27):
                passed65='<font color="green">Passed </font>'
            st65=('Enabled on <font color="blue"><b>%d</b></font>/%dregions</li>' % (numberegions, totalregions,))
        else:
            st65="Network Watcher not found"
        score65=[st65,numberegions,totalregions,passed65]
        return score65
    except Exception as e:
        logging.error("Exception in check65: %s %s" %(type(e), str(e.args)))
        st65="Failed to query for network watcher"
        passed65='<font color="orange">UNKNOWN </font>'
        score65=[st65,numberegions,totalregions,passed65]
        return score65



import os
import json
import logging
from . import az_cis_utils as az_cis_utils

def check70(subid):
    logging.info("Processing 7x...")
    chk71=""
    chk72="Disk Encryption Disabled"
    chk73="Disk Encryption Disabled"
    chk74=""
    chk75=""
    chk76=""
    passvalue71 = 0
    totalvalue71 = 0
    passvalue72 = 0
    totalvalue72 = 0
    passvalue73 = 0
    totalvalue73 = 0
    passvalue73 = 0
    totalvalue73 = 0
    passvalue74 = 0
    totalvalue74 = 0
    passvalue75 = 0
    totalvalue75 = 0
    passvalue76 = 0
    totalvalue76 = 0
    passed71='<font color="red">Failed </font>'
    passed72='<font color="red">Failed </font>'
    passed73='<font color="red">Failed </font>'
    passed74='<font color="red">Failed </font>'
    passed75='<font color="red">Failed </font>'
    passed76='<font color="green">Passed </font>'
    try:
        query70='az vm list --query "[*][resourceGroup,name]"'
        #query70=('az vm list --query "[?contains(id,\'%s\')].[resourceGroup,name]"' % subid)
        json_cis=az_cis_utils.query_az(query70)
        #i iteration per VM
        if (len(json_cis)>0):
            for i in range(len(json_cis)):
                rg = json_cis[i][0]
                vm = json_cis[i][1]
                logging.info("Checking VM: %s within RG %s",vm,rg)
                try:
                    json_cis2=az_cis_utils.query_az("az vm show -g %s --n %s --query resources" % (rg,vm,))
                    for j in range(len(json_cis2)):
                        VMExt= json_cis2[j]['virtualMachineExtensionType']
                        if ("MicrosoftMonitoringAgent" in VMExt):
                            chk71=chk71+('VM: %s %s<br></li>' % (vm,VMExt,))
                            passvalue71=passvalue71+1
                            passed71='<font color="green">Passed </font>'
                        else:
                            chk71=chk71+('VM: <b>%s</b> is missing the VM Agent<br></li>' % vm )
                        if ("Antimalware" in VMExt):
                            chk76=chk76+('VM: <b>%s</b> has End Point Protection <font color="blue">Enabled</font><br></li>\n' %vm )
                            passvalue76=passvalue76+1
                        else:
                            chk76=chk76+('VM: <b>%s</b> has End Point Protection Disabled<br></li>\n' % vm)
                            passed76='<font color="red">Failed </font>'
                        chk74=chk74+('VM: <b>%s</b> installed extenion: <font color="blue">%s</font><br></li>\n' %(vm,json_cis2[j]['virtualMachineExtensionType'],))
                except:
                    chk71=chk71+('VM: <b>%s</b> has no resources configured<br></li>\n' % vm)
                    chk74=chk74+('VM: <b>%s</b> has no extension configured<br></li>\n' % vm )
                try:
                    json_cis3=az_cis_utils.query_az("az vm encryption show --resource-group %s --name %s --query osDisk" % (rg,vm,))
                    chk72=chk72+str(json_cis3)
                except:
                    chk72=chk72+('VM: <b>%s</b> has no OSDisk encryption configured<br></li>' % vm )
                try:
                    json_cis4=az_cis_utils.query_az("az vm encryption show --resource-group %s --name %s --query dataDisk" % (rg,vm,))
                    chk73=chk73+str(json_cis4)
                except:
                    chk73=chk73+('VM: <b>%s</b> has no dataDisk encryption configured<br></li>\n' % vm)
                totalvalue71 = totalvalue71+1
                totalvalue72 = totalvalue71
                totalvalue73 = totalvalue71
                totalvalue74 = totalvalue71
                totalvalue75 = totalvalue71
                totalvalue76 = totalvalue71
            chk75=chk75+"Check Not possible via Azure CLI</li>\n"
            score71=[passvalue71,totalvalue71,passed71]
            score72=[passvalue72,totalvalue72,passed72]
            score73=[passvalue73,totalvalue73,passed73]
            score74=[passvalue74,totalvalue74,passed74]
            score75=[passvalue75,totalvalue75,passed75]
            score76=[passvalue76,totalvalue76,passed76]
            return [chk71,chk72,chk73,chk74,chk75,chk76,score71,score72,score73,score74,score75,score76]
        else:
            unkScore=[1,1,'<font color="green">Passed </font>']
            chk="No VM Found"
            return [chk,chk,chk,chk,chk,chk,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore]
    except Exception as e:
        logging.error("Exception in check62: %s %s" %(type(e), str(e.args)))
        unkScore=[0,1,'<font color="orange">UNKNOWN </font>']
        chk="Failed to query for VM"
        return [chk,chk,chk,chk,chk,chk,unkScore,unkScore,unkScore,unkScore,unkScore,unkScore]



import sys
import logging
from . import gcp_cis_utils as gcp_cis_utils
  
def check6_1_1():
    # 6.1.1 Ensure that a MySQL database instance does not allow anyone to connect with administrative privileges (Scored)

    logging.info("Not supported - 6.1.1 Ensure that a MySQL database instance does not allow anyone to connect with administrative privileges (Scored)")
    return None

def check6_1_2():
    # 6.1.2 Ensure that the 'local_infile' database flag for a Cloud SQL Mysql instance is set to 'off' (Scored)

    logging.info("6.1.2 Ensure that the 'local_infile' database flag for a Cloud SQL Mysql instance is set to 'off' (Scored)")
    details_msg = "Cloud SQL Mysql instance [%s] in project [%s] does not have 'local_infile' database flag set to 'off'"
    details = gcp_cis_utils.check_database_flag('MYSQL*', 'local_infile', 'off', details_msg)
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-6.1.1', "6.1.1 [Level 1] Ensure that the 'local_infile' database flag for a Cloud SQL Mysql instance is set to 'off' (Scored)", "\n".join(details), '4', '', '')
    return None

def check6_2_1():
    # 6.2.1 Ensure that the 'log_checkpoints' database flag for Cloud SQL PostgreSQL instance is set to 'on' (Scored)

    logging.info("6.2.1 Ensure that the 'log_checkpoints' database flag for Cloud SQL PostgreSQL instance is set to 'on' (Scored)")
    details_msg = "Cloud SQL PostgreSQL instance [%s] in project [%s] does not have 'log_checkpoints' database flag set to 'on'"
    details = gcp_cis_utils.check_database_flag("POSTGRES*", 'log_checkpoints', 'on', details_msg)
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-6.2.1', "6.2.1 [Level 1] Ensure that the 'log_checkpoints' database flag for Cloud SQL PostgreSQL instance is set to 'on' (Scored)", "\n".join(details), '4', '', '')
    return None

def check6_2_2():
    # 6.2.2 Ensure that the 'log_connections' database flag for Cloud SQL PostgreSQL instance is set to 'on' (Scored)

    logging.info("6.2.2 Ensure that the 'log_connections' database flag for Cloud SQL PostgreSQL instance is set to 'on' (Scored)")
    details_msg = "Cloud SQL PostgreSQL instance [%s] in project [%s] does not have 'log_connections' database flag set to 'on'"
    details = gcp_cis_utils.check_database_flag("POSTGRES*", 'log_connections', 'on', details_msg)
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-6.2.2', "6.2.2 [Level 1] Ensure that the 'log_connections' database flag for Cloud SQL PostgreSQL instance is set to 'on' (Scored)", "\n".join(details), '4', '', '')
    return None

def check6_2_3():
    # 6.2.3 Ensure that the 'log_disconnections' database flag for Cloud SQL PostgreSQL instance is set to 'on' (Scored)

    logging.info("6.2.3 Ensure that the 'log_disconnections' database flag for Cloud SQL PostgreSQL instance is set to 'on' (Scored)")
    details_msg = "Cloud SQL PostgreSQL instance [%s] in project [%s] does not have 'log_disconnections' database flag set to 'on'"
    details = gcp_cis_utils.check_database_flag("POSTGRES*", 'log_disconnections', 'on', details_msg)
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-6.2.3', "6.2.3 [Level 1] Ensure that the 'log_disconnections' database flag for Cloud SQL PostgreSQL instance is set to 'on' (Scored)", "\n".join(details), '4', '', '')
    return None

def check6_2_4():
    # 6.2.4 Ensure that the 'log_lock_waits' database flag for Cloud SQL PostgreSQL instance is set to 'on' (Scored)

    logging.info("6.2.4 Ensure that the 'log_lock_waits' database flag for Cloud SQL PostgreSQL instance is set to 'on' (Scored)")
    details_msg = "Cloud SQL PostgreSQL instance [%s] in project [%s] does not have 'log_lock_waits' database flag set to 'on'"
    details = gcp_cis_utils.check_database_flag("POSTGRES*", 'log_lock_waits', 'on', details_msg)
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-6.2.4', "6.2.4 [Level 1] Ensure that the 'log_lock_waits' database flag for Cloud SQL PostgreSQL instance is set to 'on' (Scored)", "\n".join(details), '4', '', '')
    return None

def check6_2_5():
    # 6.2.5 Ensure that the 'log_min_messages' database flag for Cloud SQL PostgreSQL instance is set appropriately (Not Scored)

    logging.info("6.2.5 Ensure that the 'log_min_messages' database flag for Cloud SQL PostgreSQL instance is set appropriately (Not Scored)")
    details_msg = "Cloud SQL PostgreSQL instance [%s] in project [%s] does not have 'log_min_messages' database flag set"
    details = gcp_cis_utils.check_database_flag("POSTGRES*", 'log_min_messages', None, details_msg)
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-6.2.5', "6.2.5 [Level 1] Ensure that the 'log_min_messages' database flag for Cloud SQL PostgreSQL instance is set appropriately (Not Scored)", "\n".join(details), '4', '', '')
    return None

def check6_2_6():
    # 6.2.6 Ensure that the 'log_temp_files' database flag for Cloud SQL PostgreSQL instance is set to '0' (on) (Scored)

    logging.info("6.2.6 Ensure that the 'log_temp_files' database flag for Cloud SQL PostgreSQL instance is set to '0' (on) (Scored)")
    details_msg = "Cloud SQL PostgreSQL instance [%s] in project [%s] does not have 'log_temp_files' database flag set to '0' (on)"
    details = gcp_cis_utils.check_database_flag("POSTGRES*", 'log_temp_files', '0', details_msg)
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-6.2.6', "6.2.6 [Level 1] Ensure that the 'log_temp_files' database flag for Cloud SQL PostgreSQL instance is set to '0' (on) (Scored)", "\n".join(details), '4', '', '')
    return None

def check6_2_7():
    # 6.2.7 Ensure that the 'log_min_duration_statement' database flag for Cloud SQL PostgreSQL instance is set to '-1' (disabled) (Scored)

    logging.info("6.2.7 Ensure that the 'log_min_duration_statement' database flag for Cloud SQL PostgreSQL instance is set to '-1' (disabled) (Scored)")
    details_msg = "Cloud SQL PostgreSQL instance [%s] in project [%s] does not have 'log_min_duration_statement' database flag set to '-1' (disabled)"
    details = gcp_cis_utils.check_database_flag("POSTGRES*", 'log_min_duration_statement', '-1', details_msg)
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-6.2.7', "6.2.7 [Level 1] Ensure that the 'log_min_duration_statement' database flag for Cloud SQL PostgreSQL instance is set to '-1' (disabled) (Scored)", "\n".join(details), '4', '', '')
    return None

def check6_3_1():
    # 6.3.1 Ensure that the 'cross db ownership chaining' database flag for Cloud SQL SQL Server instance is set to 'off' (Scored)

    logging.info("6.3.1 Ensure that the 'cross db ownership chaining' database flag for Cloud SQL SQL Server instance is set to 'off' (Scored)")
    details_msg = "Cloud SQL SQL Server instance [%s] in project [%s] does not have 'cross db ownership chaining' database flag set to 'off'"
    details = gcp_cis_utils.check_database_flag("SQLSERVER*", 'cross db ownership chaining', 'off', details_msg)
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-6.3.1', "6.3.1 [Level 1] Ensure that the 'cross db ownership chaining' database flag for Cloud SQL SQL Server instance is set to 'off' (Scored)", "\n".join(details), '4', '', '')
    return None

def check6_3_2():
    # 6.3.2 Ensure that the 'contained database authentication' database flag for Cloud SQL on the SQL Server instance is set to 'off' (Scored)

    logging.info("6.3.2 Ensure that the 'contained database authentication' database flag for Cloud SQL on the SQL Server instance is set to 'off' (Scored)")
    details_msg = "Cloud SQL SQL Server instance [%s] in project [%s] does not have 'contained database authentication' database flag set to 'off'"
    details = gcp_cis_utils.check_database_flag("SQLSERVER*", 'contained database authentication', 'off', details_msg)
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-6.3.2', "6.3.2 [Level 1] Ensure that the 'contained database authentication' database flag for Cloud SQL on the SQL Server instance is set to 'off' (Scored)", "\n".join(details), '4', '', '')
    return None

def check6_4():
    # 6.4 Ensure that the Cloud SQL database instance requires all incoming connections to use SSL (Scored)
    
    logging.info("6.4 Ensure that the Cloud SQL database instance requires all incoming connections to use SSL (Scored)")
    details = []
    projects = gcp_cis_utils.get_all_projects()
    for p in projects:
        out_json = gcp_cis_utils.run_gcloud_cmd("sql instances list --project=%s" % p)
        for entry in out_json:
            require_ssl = False
            settings = entry.get("settings")
            if settings is not None:
                ipConfig = settings.get("ipConfiguration")
                if ipConfig is not None:
                    req_ssl = ipConfig.get('requireSsl')
                    if req_ssl is not None and req_ssl == True:
                        require_ssl = True
            if require_ssl == False:
                details.append("Cloud SQL database instance [%s] in project [%s] does not require all incoming connections to use SSL" % (entry['name'], p))
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-6.4', "6.4 [Level 1] Ensure that the Cloud SQL database instance requires all incoming connections to use SSL (Scored)", "\n".join(details), '4', '', '')
    return None

def check6_5():
    # 6.5 Ensure that Cloud SQL database instances are not open to the world (Scored)
    
    logging.info("6.5 Ensure that Cloud SQL database instances are not open to the world (Scored)")
    details = []
    projects = gcp_cis_utils.get_all_projects()
    for p in projects:
        out_json = gcp_cis_utils.run_gcloud_cmd("sql instances list --project=%s" % p)
        for entry in out_json:
            settings = entry.get("settings")
            if settings is not None:
                ipConfig = settings.get("ipConfiguration")
                if ipConfig is not None:
                    auth_nws = ipConfig.get('authorizedNetworks')
                    if auth_nws is None:
                        continue
                    for auth_nw in auth_nws:
                        if auth_nw.get('value') == "0.0.0.0/0":
                            details.append("Cloud SQL database instance [%s] in project [%s] is open to the world" % (entry['name'], p))
                            break
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-6.5', "6.5 [Level 1] Ensure that Cloud SQL database instances are not open to the world (Scored)", "\n".join(details), '4', '', '')
    return None

def check6_6():
    # 6.6 Ensure that Cloud SQL database instances do not have public IPs (Scored)
    
    logging.info("6.6 Ensure that Cloud SQL database instances do not have public IPs (Scored)")
    details = []
    projects = gcp_cis_utils.get_all_projects()
    for p in projects:
        out_json = gcp_cis_utils.run_gcloud_cmd("sql instances list --project=%s" % p)
        for entry in out_json:
            if entry['instanceType'] == "CLOUD_SQL_INSTANCE" and entry['backendType'] == "SECOND_GEN":
                ipAddresses = entry.get('ipAddresses')
                if ipAddresses is not None:
                    for ipAddress in ipAddresses:
                        if ipAddress['type'] == "PRIMARY":
                            details.append("Cloud SQL database instance [%s] in project [%s] has Public IP address" % (entry['name'], p))
                            break
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-6.6', "6.6 [Level 2] Ensure that Cloud SQL database instances do not have public IPs (Scored)", "\n".join(details), '5', '', '')
    return None

def check6_7():
    # 6.7 Ensure that Cloud SQL database instances are configured with automated backups (Scored)
    
    logging.info("6.7 Ensure that Cloud SQL database instances are configured with automated backups (Scored)")
    details = []
    projects = gcp_cis_utils.get_all_projects()
    for p in projects:
        out_json = gcp_cis_utils.run_gcloud_cmd("sql instances list --project=%s" % p)
        for entry in out_json:
            backup_enabled = False
            settings = entry.get("settings")
            if settings is not None:
                backupConfig = settings.get("backupConfiguration")
                if backupConfig is not None:
                    bkp_enabled = backupConfig.get('enabled')
                    if bkp_enabled is not None and bkp_enabled == True:
                        backup_enabled = True
            if backup_enabled == False:
                details.append("Cloud SQL database instance [%s] in project [%s] does not have automated backups enabled" % (entry['name'], p))
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-6.7', "6.7 [Level 1] Ensure that Cloud SQL database instances are configured with automated backups (Scored)", "\n".join(details), '4', '', '')
    return None

def run_checks():
    config_issues = []
    gcp_cis_utils.append_issue(config_issues, check6_1_1())
    gcp_cis_utils.append_issue(config_issues, check6_1_2())
    gcp_cis_utils.append_issue(config_issues, check6_2_1())
    gcp_cis_utils.append_issue(config_issues, check6_2_2())
    gcp_cis_utils.append_issue(config_issues, check6_2_3())
    gcp_cis_utils.append_issue(config_issues, check6_2_4())
    gcp_cis_utils.append_issue(config_issues, check6_2_5())
    gcp_cis_utils.append_issue(config_issues, check6_2_6())
    gcp_cis_utils.append_issue(config_issues, check6_2_7())
    gcp_cis_utils.append_issue(config_issues, check6_3_1())
    gcp_cis_utils.append_issue(config_issues, check6_3_2())
    gcp_cis_utils.append_issue(config_issues, check6_4())
    gcp_cis_utils.append_issue(config_issues, check6_5())
    gcp_cis_utils.append_issue(config_issues, check6_6())
    gcp_cis_utils.append_issue(config_issues, check6_7())
    return config_issues




import sys
import os
import subprocess
import logging
import json

from . import utils
from .gcp_cis_tool import gcp_cis_utils
from . import docker

g_encoding = None

def set_encoding(encoding):
    global g_encoding
    g_encoding = encoding

def get_encoding():
    global g_encoding
    return g_encoding

def run_az_cmd(cmd):
    cmd = 'az ' + cmd + ' --output json --only-show-errors'
    try:
        logging.debug("Running cmd [%s]", cmd)
        cmd_output = subprocess.check_output([cmd], shell=True, stdin=None, stderr=None)
        cmd_output = cmd_output.decode(get_encoding())
        ret_json = json.loads(cmd_output)
    except subprocess.CalledProcessError:
        logging.error("Error running az command [%s]", cmd)
        sys.exit(1)
    except ValueError:
        logging.error("Error parsing JSON output for az command [%s]: %s", cmd, cmd_output)
        sys.exit(1)
    return ret_json

def get_latest_tag(image_name, repository):
    tags_cmd = "acr repository show-tags --name " + repository + " --repository " + image_name + " --orderby time_desc"
    t_json = run_az_cmd(tags_cmd)
    if len(t_json) > 0:
        if "latest" in t_json:
            return ":latest"
        else:
            return ":" + t_json[0]
    return None

def get_inventory(args):
    set_encoding(args.encoding)
    allassets = [] 
    if args.registry is None and args.image is None:
        logging.error("Either fully qualified image name (with repository and tag / digest) or repository url needs to be specified")
        return None
    if not args.image:
        logging.info("Starting discovery of images in ACR")
        getLoginServer_cmd = "acr show --name " + args.registry
        ret_json = run_az_cmd(getLoginServer_cmd)
        loginServer = ret_json['loginServer']
        ilist_cmd = "acr repository list --name " + args.registry
        i_json = run_az_cmd(ilist_cmd)
        logging.info("Found %d images in %s", len(i_json), args.registry)
        for i in i_json:
            tag = get_latest_tag(i, args.registry)
            if tag == None:
                logging.error("Unable to determine latest tag for image. Skipping %s",i)
                continue
            logging.info("Using tag/digest '"+tag[1:]+"'")
            image_name = loginServer + "/" + i + tag
            args.image = image_name
            args.assetid = image_name
            args.assetid = args.assetid.replace('/','-')
            args.assetid = args.assetid.replace(':','-')
            args.assetname = image_name
            logging.info("Discovering image "+args.image)
            assets = docker.get_inventory(args)
            if assets:
                allassets = allassets + assets
        for a in allassets:
            a['tags'].append('ACR')
        return allassets
    else:
        tokens = args.image.split('.azurecr.io/')
        if len(tokens) != 2:
            logging.error("Invalid ACR image name specified. Please specify fully qualified image name ( like myregistry.azurecr.io/myimage[:mytag] )")
            return None
        repository = tokens[0]
        image_name = tokens[1]
        if ':' not in image_name:
            tag = get_latest_tag(image_name, repository)
            if tag == None:
                logging.error("Unable to determine latest tag / digest for image")
                return None 
            logging.info("Using tag/digest '"+tag[1:]+"'")
            args.image = args.image + tag
        args.assetid = args.image
        args.assetid = args.assetid.replace('/','-')
        args.assetid = args.assetid.replace(':','-')
        args.assetname = args.image
        logging.info("Discovering image "+args.image)
        assets = docker.get_inventory(args)
        if assets != None:
            for a in assets:
                a['tags'].append('ACR')
        return assets 




import sys
import platform
import os
import logging
import json
import csv
import PyPDF4
from pdfminer.pdfparser import PDFParser
from pdfminer.pdfdocument import PDFDocument
from pdfminer.pdfpage import PDFPage
from pdfminer.pdfpage import PDFTextExtractionNotAllowed
from pdfminer.pdfinterp import PDFResourceManager
from pdfminer.pdfinterp import PDFPageInterpreter
from pdfminer.converter import PDFPageAggregator
from pdfminer.layout import LAParams, LTTextBox, LTTextLine, LTText

from . import utils

all_asset_types = None

def get_products_from_pdf_file_using_pypdf(in_file):
    pdf_fd = open(in_file, 'rb')
    pdfFileReader = PyPDF4.PdfFileReader(pdf_fd)
    complete_text = ''
    for page_num in range(pdfFileReader.numPages):
        pdfPage = pdfFileReader.getPage(page_num)
        page_text = pdfPage.extractText()
        complete_text = complete_text + page_text
    products = []
    join = False
    for i in complete_text.split('\n'):
        for j in i.split(','):
            j = j.strip()
            if len(j) > 0 and j != '!' and j.isdigit() == False:
                j = j.replace(u'\xde','fi')
                if join == True:
                    join = False
                    products[-1] = products[-1] + j
                    continue
                if j == '"':
                    join = True
                    products[-1] = products[-1] + 'ff'
                    continue
                products.append(j)
    return products

def get_products_from_pdf_file_using_pdfminer(in_file):
    logging.getLogger("pdfminer").setLevel(logging.WARNING)
    products = []
    fp = open(in_file, 'rb')
    parser = PDFParser(fp)
    password = ''
    document = PDFDocument(parser, password)
    # Check if the document allows text extraction. If not, abort.
    if not document.is_extractable:
        fp.close()
        logging.error("PDF document [%s] does not allow Text Extraction", in_file)
        return products
    rsrcmgr = PDFResourceManager()
    laparams = LAParams()
    device = PDFPageAggregator(rsrcmgr, laparams=laparams)
    interpreter = PDFPageInterpreter(rsrcmgr, device)

    complete_text = ''
    for page in PDFPage.create_pages(document):
        interpreter.process_page(page)
        layout = device.get_result()
        for lt_obj in layout:
            if isinstance(lt_obj, LTTextBox) or isinstance(lt_obj, LTTextLine):
                complete_text = complete_text + lt_obj.get_text()

    for i in complete_text.split('\n'):
        for j in i.split(','):
            j = j.strip()
            products.append(j)
    return products

def get_assets_from_json_file(in_file):
    assets = []
    with open(in_file,'r') as fd:
        try:
            assets = json.load(fd)
        except ValueError:
            logging.error("Error loading JSON file [%s]", in_file)
            sys.exit(1)
    return assets

def get_assets_from_csv_file(in_file, args):
    assets = []
    with open(in_file, 'rU') as fd:
        csv_reader = csv.DictReader(fd, delimiter=',', escapechar='\\')
        for record in csv_reader:
            asset = create_asset_from_csv_record(record, args)
            if asset is not None:
                assets.append(asset)
    return assets

def create_asset_from_csv_record(record, args):
    asset = { }
    asset['id'] = record['Asset Id'].strip()
    asset['name'] = record['Asset Name'].strip()
    asset['type'] = record['Asset Type'].strip()
    asset['owner'] = record['Owner'].strip()
    asset['tags'] = get_mv_field(record['Tags'])
    asset['products'] = get_mv_field(record['Products'])
    asset['patches'] = get_mv_field(record['Patches'])

    if validate_update_csv_asset(asset, args) == False:
        return None

    return asset

def get_mv_field(mv_field_value):
    mv_field_values = mv_field_value.split(';')
    ret_val = []
    for item in mv_field_values:
        item = item.strip()
        if item == '':
            continue
        ret_val.append(item)
    return ret_val

def validate_update_csv_asset(asset, args):
    global all_asset_types
    if asset['id'] == '':
        return False
    if asset['name'] == '':
        return False
    if asset['type'] == '':
        asset['type'] = 'Other'
    else:
        if all_asset_types is None:
            url = "https://" + args.instance + "/api/v1/assets/types"
            auth_data = "?handle=" + args.handle + "&token=" + args.token + "&format=json"
            response = utils.requests_get(url + auth_data)
            if response.status_code != 200:
                logging.error("Unable to get valid asset types.")
                sys.exit(1)
            all_asset_types = response.json()
        if asset['type'] not in all_asset_types:
            logging.warning("Skipping asset [%s] with invalid asset type [%s]", asset['name'],asset['type'])
            return False
    if asset['owner'] == '':
        asset['owner'] = args.handle
    return True

def enumerate_files(in_path, file_ext):
    ret_files = []
    for root, subdirs, files in os.walk(in_path):
        for fname in files:
            file_path = os.path.join(root, fname)
            if file_path.endswith(file_ext):
                ret_files.append(file_path)
    return ret_files

def check_and_update_scan(args, assets):
    # if user has indicated no_scan then honor it
    if args.no_scan == True:
        return
    for a in assets:
        if a.get('compliance_metadata') is not None:
            args.mode = "file_repo" # update the mode to help trigger license scan

def get_inventory(args):
    in_file = args.input

    if os.path.isdir(in_file):
        logging.info("Processing CSV and JSON files in specified directory [%s]", in_file)
        json_files = enumerate_files(in_file, '.json')
        assets = []
        for json_file in json_files:
            logging.info("Retriving products from JSON file [%s]", json_file)
            temp_assets = get_assets_from_json_file(json_file)
            assets.extend(temp_assets)
        csv_files = enumerate_files(in_file, '.csv')
        for csv_file in csv_files:
            logging.info("Ingesting CSV file [%s]", csv_file)
            temp_assets = get_assets_from_csv_file(csv_file, args)
            assets.extend(temp_assets)
        check_and_update_scan(args, assets)
        return assets
    elif os.path.isfile(in_file) == False:
        logging.error("Error specified file [%s] not found...", in_file)
        sys.exit(1)
    temp = in_file.rfind('.')
    if temp == -1:
        logging.error('Error specified file [%s] is missing extension...', in_file)
        sys.exit(1)
    in_file_ext = in_file[temp+1:]
    if in_file_ext == 'pdf':
        logging.info("Retriving products from PDF file [%s]", in_file)
        products = get_products_from_pdf_file_using_pypdf(in_file)
        if len(products) == 0:
            products = get_products_from_pdf_file_using_pdfminer(in_file)
        logging.info("Done retriving products from PDF file")
    elif in_file_ext == 'json':
        logging.info("Retriving products from JSON file [%s]", in_file)
        assets = get_assets_from_json_file(in_file)
        check_and_update_scan(args, assets)
        return assets
    elif in_file_ext == 'csv':
        logging.info("Ingesting CSV file [%s]", in_file)
        assets = get_assets_from_csv_file(in_file, args)
        check_and_update_scan(args, assets)
        return assets
    else:
        logging.error('Error unsupported input file type [%s] specified! Supported file types are JSON and PDF.', in_file_ext)
        sys.exit(1)
    
    if args.assetid is None:
        asset_id = os.path.basename(in_file)
        temp = asset_id.rfind('.')
        asset_id = asset_id[:temp] # remove the extension
    else:
        asset_id = args.assetid
    asset_id = asset_id.replace(' ','-')
    asset_id = asset_id.replace('/','-')
    asset_id = asset_id.replace(':','-')
    if args.assetname is not None:
        asset_name = args.assetname.replace('/','-')
        asset_name = asset_name.replace(':','-')
    else:
        asset_name = asset_id

    asset = { }
    asset['id'] = asset_id
    asset['name'] = asset_name
    if args.type == 'repo':
        asset['type'] = 'Source Repository'
    else:
        asset['type'] = 'Other'
    asset['owner'] = args.handle
    asset['products'] = products
    assets = [ asset ]
    check_and_update_scan(args, assets)
    return assets




import sys
import json
import boto3
import codecs
import os
import logging
import tempfile

RELEVANT_BUCKET_OBJECT_KEYS = ['AWS:WindowsUpdate', 'AWS:Application', 'AWS:InstanceInformation']

class AWS(object):
    def __init__(self, params):
        self.bucket = params['bucket']
        self.account_id = params['account_id']
        self.access_key = params['access_key']
        self.secret_key = params['secret_key']
        self.region = params['region']
        self.enable_tracking_tags = params['enable_tracking_tags']

    def auth(self, url, user, passwd):
        raise NotImplementedError("Subclass must implement abstract method")

    def oauth(self, url, user, passwd):
        raise NotImplementedError("Subclass must implement abstract method")

class EC2Impl(AWS):
    def __init__(self, params):
        super(EC2Impl, self).__init__(params)
        self.populate_bucket_object_list()

    def auth(self):
        raise NotImplementedError("Not implemented")

    def oauth(self, url, user, passwd):
        raise NotImplementedError("Not implemented")

    def configure():
        #write to the file(s), ~/.aws/config and ~/.aws/credentials
        raise NotImplementedError("Not implemented")

    def populate_bucket_object_list(self):
        logging.info("Obtainining asset inventory details from S3 bucket (this might take some time)...")
        self.bucket_object_list = []
        session = boto3.session.Session(aws_access_key_id=self.access_key,aws_secret_access_key=self.secret_key,region_name=self.region)
        s3 = session.resource('s3')
        self.s3_bucket = s3.Bucket(self.bucket)
        for obj in self.s3_bucket.objects.all():
            for b_key in RELEVANT_BUCKET_OBJECT_KEYS:
                if b_key in obj.key:
                    self.bucket_object_list.append(obj)
        #print "S3_relevant_bucket_object_array_len: %s" % (str(len(self.bucket_object_list)))
        #print "S3_relevant_bucket_objecgt: %s" % (self.bucket_object_list)

    def windows_patch_inventory(self, host):
        for obj in self.bucket_object_list:
            if 'AWS:WindowsUpdate' in obj.key and host in obj.key:
                splits = obj.key.rsplit('/')
                fname = tempfile.gettempdir() + os.path.sep + 'WindowsUpdate-' +  splits[-1]
                if os.path.isfile(fname) == True:
                    os.remove(fname)
                self.s3_bucket.download_file(obj.key,fname)
                try:
                    data = []
                    with codecs.open(fname,'rU','utf-8') as f:
               	        for line in f:
                            data.append(json.loads(line))
                    patches = []
                    for i in range(len(data)):
                        patch = {}
                        patch['url'] = ''
                        patch['id'] = data[i]['HotFixId']
                        patch['product'] = ''
                        patch['description'] = ''
                        patches.append(patch)
                    return patches
                except ValueError:
                    logging.error("JSON parsing failed for [%s]", fname)
                try:
                    os.remove(fname)
                except OSError:
                    logging.error("Failed to cleanup file [%s]", fname)
        return None

    def product_inventory(self, host, host_type):
        for obj in self.bucket_object_list:
            if 'AWS:Application' in obj.key and host in obj.key:
                splits = obj.key.rsplit('/')
                fname = tempfile.gettempdir() + os.path.sep + 'Application-' +  splits[-1]
                if os.path.isfile(fname) == True:
                    os.remove(fname)
                self.s3_bucket.download_file(obj.key,fname)
                try:
                    data = []
                    with codecs.open(fname,'rU','utf-8') as f:
               	        for line in f:
                            data.append(json.loads(line))
                    #print data
                    products = []
                    for i in range(len(data)):
                        if "Windows" in host_type:
                            pname = data[i]['Name']
                            pversion = data[i]['Version']
                            products.append(pname+' '+pversion)
                        elif host_type == "Amazon Linux AMI":
                            pname = data[i]['Name']
                            pver = data[i]['Version']
                            prpm = data[i]['PackageId']
                            parch = data[i]['Architecture']
                            index = prpm.rfind('-')
                            temp = prpm[index:-8]
                            product_name = pname+' '+pver+temp+"."+parch
                            #print "[%s][%s] ==> [%s]" % (pname, prpm, product_name)
                            products.append(product_name)
                        elif host_type == "Ubuntu":
                            pname = data[i]['Name']
                            pver = data[i]['Version']
                            parch = data[i]['Architecture']
                            product_name = pname+' '+pver
                            #print "[%s][%s] ==> [%s]" % (pname, prpm, product_name)
                            products.append(product_name)
                    return products
                except ValueError:
                    logging.error("JSON parsing failed for [%s]", fname)
                try:
                    os.remove(fname)
                except OSError:
                    logging.error("Failed to cleanup file [%s]", fname)
        return None

    def get_asset_type(self, platformName):
        if platformName == "Amazon Linux":
            return "Amazon Linux AMI"
        return platformName
      
    def asset_inventory(self, email):
        logging.info("Compiling asset inventory...")
        assets = []
        for obj in self.bucket_object_list:
            if 'AWS:InstanceInformation' in obj.key:
                splits = obj.key.rsplit('/')
                fname = tempfile.gettempdir() + os.path.sep + 'InstanceInformation-' +  splits[-1]
                if os.path.isfile(fname) == True:
                    os.remove(fname)
                self.s3_bucket.download_file(obj.key,fname)
                try:
                    data = []
                    with codecs.open(fname,'rU','utf-8') as f:
                        s = f.read()
                        #print s
                        data.append(json.loads(s))
                    if data[0].get('InstanceStatus') == 'Terminated':
                        # skip instances which are not running
                        continue
                    asset = {}
                    asset['id'] = data[0]['resourceId']
                    asset['name'] = data[0]['ComputerName']
                    logging.info("Found asset [%s] in AWS inventory", asset['name'])
                    asset['type'] = self.get_asset_type(data[0]['PlatformName'])
                    asset['owner'] = email
                    os_release = None
                    if 'Linux' in asset['type']:
                        asset['tags'] = [ 'Linux' ]
                    elif 'Ubuntu' in asset['type']:
                        asset['tags'] = [ 'Linux' ]
                        os_release = asset['type'] + " " + data[0]['PlatformVersion']
                    elif 'Windows' in asset['type']:
                        asset['tags'] = [ 'Windows' ]
                    if os_release is not None:
                        asset['tags'].append("OS_RELEASE:" + os_release)
                    if self.enable_tracking_tags == True:
                        asset['tags'].append("SOURCE:AWS:"+self.account_id)
                    else:
                        asset['tags'].append("SOURCE:AWS")
                    logging.info("Retrieving product details for [%s]", asset['name'])
                    asset['products'] = self.product_inventory(asset['id'], asset['type'])
                    logging.info("Retrieving patch details for [%s]", asset['name'])
                    asset['patches'] = self.windows_patch_inventory(asset['id'])
                    assets.append(asset)
                except ValueError:
                    logging.error("JSON parsing failed for [%s]", fname)
                try:
                    os.remove(fname)
                except OSError:
                    logging.error("Failed to cleanup file [%s]", fname)
        logging.info("Total %s assets found in inventory...",str(len(assets)))
        return assets
         
def get_inventory(args):
    params =  {}
    params['account_id'] = args.aws_account
    params['access_key'] = args.aws_access_key
    params['secret_key'] = args.aws_secret_key
    params['region'] = args.aws_region
    params['bucket'] = args.aws_s3_bucket
    params['enable_tracking_tags'] = args.enable_tracking_tags
    aws =  EC2Impl(params)
    assets = aws.asset_inventory(args.handle)
    return assets



#!/usr/bin/env python
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.

import sys
import os
import logging
import argparse
import time
import json
import traceback
import pkg_resources
import pkgutil
import importlib
import getpass
from os.path import expanduser

from . import aws
from . import linux
from . import repo
from . import docker
from . import azure
from . import acr
from . import ecr
from . import gcp
from . import gcr
from . import servicenow
from . import inv_file
from . import fingerprint
from . import dast 
from . import docker_cis
from . import aws_cis
from . import azure_cis
from . import gcp_cis
from . import utils
from . import policy as policy_lib
from .__init__ import __version__


def export_assets_to_file(assets, json_file):
    logging.info("Exporting assets to JSON file [%s]", json_file)
    with open(json_file, "w") as fd:
        json.dump(assets, fd, indent=2, sort_keys=True)
    logging.info("Successfully exported assets to JSON file!")

def push_asset_to_TW(asset, args):
    asset_url = "https://" + args.instance + "/api/v2/assets/"
    auth_data = "?handle=" + args.handle + "&token=" + args.token + "&format=json"
    if args.email_report:
        auth_data = auth_data + "&esr=true" # email secrets report (esr)
    asset_id = asset['id']

    resp = utils.requests_get(asset_url + asset_id + "/" + auth_data)
    if resp.status_code != 200:
        logging.info("Creating new asset [%s]", asset_id)
        # Asset does not exist so create one with POST
        resp = utils.requests_post(asset_url + auth_data, json=asset)
        if resp.status_code == 200:
            logging.info("Successfully created new asset [%s]", asset_id)
            logging.info("Response content: %s", resp.content.decode(args.encoding))
            return asset_id, True
        else:
            logging.error("Failed to create new asset [%s]", asset_id)
            logging.error("Response details: %s", resp.content.decode(args.encoding))
            return None, False
    else:
        logging.info("Updating asset [%s]", asset_id)
        # asset exists so update it with PUT
        resp = utils.requests_put(asset_url + asset_id + "/" + auth_data, json=asset)
        if resp.status_code == 200:
            logging.info("Successfully updated asset [%s]", asset_id)
            logging.debug("Response content: %s", resp.content.decode(args.encoding))
            resp_json = resp.json()
            if 'No product updates' in resp_json['status']:
                return asset_id, False
            return asset_id, True
        else:
            logging.error("Failed to update existing asset [%s]", asset_id)
            logging.error("Response details: %s", resp.content.decode(args.encoding))
            return None, False

def push_assets_to_TW(assets, args):
    asset_id_list = []
    scan_asset_id_list = []
    for asset in assets:
        asset_id, scan = push_asset_to_TW(asset, args)
        if asset_id is not None:
            asset_id_list.append(asset_id)
        if scan:
            scan_asset_id_list.append(asset_id)
    return asset_id_list, scan_asset_id_list

def run_scan(asset_id_list, pj_json, args):
    if args.no_scan is not True:
        if len(asset_id_list) == 0:
            logging.debug("No asset scan required")
            return 
        run_va_scan = True
        run_lic_scan = True
        if pj_json is not None:
            for policy in pj_json['policy_json']:
                if policy['type'] == 'vulnerability':
                    logging.info("Impact assessment performed as part of policy evaluation")
                    run_va_scan = False # VA scan already done, so don't do it again
                elif policy['type'] == 'license':
                    logging.info("License compliance performed as part of policy evaluation")
                    run_lic_scan = False # License scan already done, so don't do it again

        scan_api_url = "https://" + args.instance + "/api/v1/scans/?handle=" + args.handle + "&token=" + args.token + "&format=json"
        if run_va_scan:
            # Start VA
            scan_payload = { }
            scan_payload['scan_type'] = 'full' 
            scan_payload['assets'] = asset_id_list
            # if args.purge_assets:
            #    scan_payload['mode'] = 'email-purge'
            if args.email_report:
                scan_payload['mode'] = 'email'
            resp = utils.requests_post(scan_api_url, json=scan_payload)
            if resp.status_code == 200:
                logging.info("Started impact refresh")
            else:
                logging.error("Failed to start impact refresh")
                logging.error("Response details: %s", resp.content.decode(args.encoding))
        if run_lic_scan and (args.mode == "repo" or args.mode == "file_repo"):
            # Start license compliance assessment
            scan_payload = { }
            scan_payload['assets'] = asset_id_list
            scan_payload['license_scan'] = True
            # if args.purge_assets:
            #    scan_payload['mode'] = 'email-purge'
            if args.email_report:
                scan_payload['mode'] = 'email'
            resp = utils.requests_post(scan_api_url, json=scan_payload)
            if resp.status_code == 200:
                logging.info("Started license compliance assessment")
            else:
                logging.error("Failed to start license compliance assessment")
                logging.error("Response details: %s", resp.content.decode(args.encoding))

def add_asset_tags(assets, tags):
    for asset in assets:
        existing_tags = asset.get('tags')
        if existing_tags is None:
            asset['tags'] = tags
        else:
            existing_tags.extend(tags)

def add_asset_criticality_tag(assets, asset_criticality):
    asset_criticality_tag = 'CRITICALITY:'+str(asset_criticality)
    add_asset_tags(assets, [asset_criticality_tag])

def sub_pkg_get_inventory(args):
    dist = "twigs_" + args.mode
    try:
        pkg_resources.get_distribution(dist)
    except pkg_resources.DistributionNotFound:
        logging.error("Error required package [%s] is not installed", dist)
        logging.error("Please install using command [sudo pip install %s]", dist)
        sys.exit(1)
    module = importlib.import_module('%s.%s' % (dist,dist))
    get_inventory_func = getattr(module, "get_inventory")
    assets = get_inventory_func(args)
    return assets

def authenticate_user(tw_user, tw_pwd, tw_instance):
    payload = { }
    payload["handle"] = tw_user
    payload["password"] = tw_pwd
    twigs_auth_url = "https://" + tw_instance + "/api/v1/twigsauth/"
    resp = utils.requests_post(twigs_auth_url, json=payload)
    if resp.status_code == 200:
        logging.info("User logged in successfully")
        return resp.json()["token"]
    else:
        logging.error("User authentication failed")
        logging.debug("Response: %s", resp.content)
        sys.exit(1)

def login_user(args):
    try:
        if sys.version_info.major < 3:
            tw_user = raw_input("Enter email of ThreatWatch user: ")
        else:
            tw_user = input("Enter email of ThreatWatch user: ")
        temp_pwd = getpass.getpass(prompt='Enter password: ')
        if sys.version_info.major < 3:
            tw_instance = raw_input("Enter ThreatWatch instance [threatwatch.io]: ")
        else:
            tw_instance = input("Enter ThreatWatch instance [threatwatch.io]: ")
    except KeyboardInterrupt:
        print("")
        sys.exit(1)
    if tw_instance == "":
        tw_instance = "threatwatch.io"
    tw_token = authenticate_user(tw_user, temp_pwd, tw_instance)
    auth_dict = {}
    auth_dict['handle'] = tw_user
    auth_dict['token'] = tw_token
    auth_dict['instance'] = tw_instance
    user_home_dir = expanduser("~")
    tw_dir = user_home_dir + os.path.sep + '.tw'
    if os.path.isdir(tw_dir) == False:
        os.mkdir(tw_dir, 0o700)
    auth_file = tw_dir + os.path.sep + 'auth.json'
    if os.path.exists(auth_file):
        os.remove(auth_file)
    with open(auth_file, "w") as fd:
        json.dump(auth_dict, fd)
    os.chmod(auth_file, 0o600)
    sys.exit(0)

def logout_user(args):
    user_home_dir = expanduser("~")
    tw_dir = user_home_dir + os.path.sep + '.tw'
    if os.path.isdir(tw_dir):
        auth_file = tw_dir + os.path.sep + 'auth.json'
        if os.path.exists(auth_file):
            os.remove(auth_file)
    sys.exit(0)

def get_logged_in_user_details():
    ret_dict = { }
    user_home_dir = expanduser("~")
    tw_dir = user_home_dir + os.path.sep + '.tw'
    if os.path.isdir(tw_dir):
        auth_file = tw_dir + os.path.sep + 'auth.json'
        if os.path.exists(auth_file):
            with open(auth_file, "r") as fd:
                ret_dict = json.load(fd)
    return ret_dict

def main(args=None):

    try:
    
        if args is None:
            args = sys.argv[1:]

        if sys.platform != 'win32':
            utils.set_requests_verify(os.path.dirname(os.path.realpath(__file__)) + os.sep + 'gd-ca-bundle.crt')
        logfilename = "twigs.log"
        logging_level = logging.WARN

        parser = argparse.ArgumentParser(description='ThreatWatch Information Gathering Script (twigs) to discover assets like hosts, cloud instances, containers and project repositories')
        subparsers = parser.add_subparsers(title="modes", description="Discovery modes supported", dest="mode")
        # Required arguments
        parser.add_argument('--version', action='version', version='%(prog)s ' + __version__)
        parser.add_argument('--handle', help='The ThreatWatch registered email of the user. Note this can set as "TW_HANDLE" environment variable', required=False)
        parser.add_argument('--token', help='The ThreatWatch API token of the user. Note this can be set as "TW_TOKEN" environment variable', required=False)
        parser.add_argument('--instance', help='The ThreatWatch instance. Note this can be set as "TW_INSTANCE" environment variable')
        parser.add_argument('--tag_critical', action='store_true', help='Tag the discovered asset(s) as critical')
        parser.add_argument('--tag', action='append', help='Add specified tag to discovered asset(s). You can specify this option multiple times to add multiple tags')
        #parser.add_argument('--asset_criticality', choices=['1', '2', '3','4', '5'], help='Business criticality of the discovered assets on a scale of 1 (low) to 5 (high).', required=False)
        parser.add_argument('--apply_policy', help='One or more policy names as a comma-separated list', required=False)
        parser.add_argument('--out', help='Specify name of the JSON file to hold the exported asset information.')
        parser.add_argument('--no_scan', action='store_true', help='Do not initiate a baseline assessment')
        parser.add_argument('--email_report', action='store_true', help='After impact refresh is complete email scan report to self')
        group = parser.add_mutually_exclusive_group()
        group.add_argument('-q','--quiet', action='store_true', help='Disable verbose logging')
        group.add_argument('-v','--verbosity', action='count', default=0, help='Specify the verbosity level. Use multiple times to increase verbosity level')
        if sys.platform != 'win32':
            parser.add_argument('--schedule', help='Run this twigs command at specified schedule (crontab format)')
        parser.add_argument('--encoding', help='Specify the encoding. Default is "latin-1"', default='latin-1')
        parser.add_argument('--insecure', action='store_true', help=argparse.SUPPRESS)
        # parser.add_argument('--purge_assets', action='store_true', help='Purge the asset(s) after impact refresh is complete and scan report is emailed to self')


        # Arguments required for Login
        parser_login = subparsers.add_parser("login", help = "Login to twigs")

        # Arguments required for Logout
        parser_logout = subparsers.add_parser("logout", help = "Logout from twigs")

        # Arguments required for AWS discovery
        parser_aws = subparsers.add_parser ("aws", help = "Discover AWS instances")
        parser_aws.add_argument('--aws_account', help='AWS account ID', required=True)
        parser_aws.add_argument('--aws_access_key', help='AWS access key', required=True)
        parser_aws.add_argument('--aws_secret_key', help='AWS secret key', required=True)
        parser_aws.add_argument('--aws_region', help='AWS region', required=True)
        parser_aws.add_argument('--aws_s3_bucket', help='AWS S3 inventory bucket', required=True)
        parser_aws.add_argument('--enable_tracking_tags', action='store_true', help='Enable recording AWS specific information (like AWS Account ID, etc.) as asset tags', required=False)

        # Arguments required for AWS Container Registry discovery 
        parser_ecr = subparsers.add_parser ("ecr", help = "Discover AWS Container Registry (ECR) container images")
        parser_ecr.add_argument('--registry', help='The AWS Container Registry (AWS account ID) which needs to be inspected for all repositories.')
        parser_ecr.add_argument('--image', help='The fully qualified image name (repositoryUri with optional tag) to be inspected. If tag is not given, latest will be determined for all images under this repository')
        parser_ecr.add_argument ("--repository_type", help = "Specify repository type (public/private). Defaults to private repositories if not specified", choices=['public','private'], default='private')
        parser_ecr.add_argument('--tmp_dir', help='Temporary directory. Defaults to /tmp', required=False)

        parser_ecr.add_argument('--containerid', help=argparse.SUPPRESS, required=False)
        parser_ecr.add_argument('--assetid', help=argparse.SUPPRESS, required=False)
        parser_ecr.add_argument('--assetname', help=argparse.SUPPRESS, required=False)
        parser_ecr.add_argument('--start_instance', action='store_true', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--repo', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--type', choices=repo.SUPPORTED_TYPES, help=argparse.SUPPRESS)
        parser_ecr.add_argument('--level', help=argparse.SUPPRESS, choices=['shallow','deep'], default='shallow')
        parser_ecr.add_argument('--secrets_scan', action='store_true', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--enable_entropy', action='store_true', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--regex_rules_file', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--check_common_passwords', action='store_true', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--common_passwords_file', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--include_patterns', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--include_patterns_file', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--exclude_patterns', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--exclude_patterns_file', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--mask_secret', action='store_true', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--no_code', action='store_true', help=argparse.SUPPRESS)
        parser_ecr.add_argument('--sast', action='store_true', help=argparse.SUPPRESS)

        # Arguments required for Azure discovery
        parser_azure = subparsers.add_parser ("azure", help = "Discover Azure instances")
        parser_azure.add_argument('--azure_tenant_id', help='Azure Tenant ID', required=True)
        parser_azure.add_argument('--azure_application_id', help='Azure Application ID', required=True)
        parser_azure.add_argument('--azure_application_key', help='Azure Application Key', required=True)
        parser_azure.add_argument('--azure_subscription', help='Azure Subscription. If not specified, then available values will be displayed', required=False)
        parser_azure.add_argument('--azure_resource_group', help='Azure Resource Group. If not specified, then available values will be displayed', required=False)
        parser_azure.add_argument('--azure_workspace', help='Azure Workspace ID. If not specified, then available values will be displayed', required=False)
        parser_azure.add_argument('--enable_tracking_tags', action='store_true', help='Enable recording Azure specific information (like Azure Tenant ID, etc.) as asset tags', required=False)

        # Arguments required for Azure Container Registry container discovery 
        parser_acr = subparsers.add_parser ("acr", help = "Discover Azure Container Registry (ACR) container images")
        parser_acr.add_argument('--registry', help='The Azure Container Registry which needs to be inspected.')
        parser_acr.add_argument('--image', help='The fully qualified image name (with tag) which needs to be inspected. If tag is not given, latest will be determined and used.')
        parser_acr.add_argument('--tmp_dir', help='Temporary directory. Defaults to /tmp', required=False)
        parser_acr.add_argument('--containerid', help=argparse.SUPPRESS, required=False)
        parser_acr.add_argument('--assetid', help=argparse.SUPPRESS, required=False)
        parser_acr.add_argument('--assetname', help=argparse.SUPPRESS, required=False)
        parser_acr.add_argument('--start_instance', action='store_true', help=argparse.SUPPRESS)
        parser_acr.add_argument('--repo', help=argparse.SUPPRESS)
        parser_acr.add_argument('--type', choices=repo.SUPPORTED_TYPES, help=argparse.SUPPRESS)
        parser_acr.add_argument('--level', help=argparse.SUPPRESS, choices=['shallow','deep'], default='shallow')
        parser_acr.add_argument('--secrets_scan', action='store_true', help=argparse.SUPPRESS)
        parser_acr.add_argument('--enable_entropy', action='store_true', help=argparse.SUPPRESS)
        parser_acr.add_argument('--regex_rules_file', help=argparse.SUPPRESS)
        parser_acr.add_argument('--check_common_passwords', action='store_true', help=argparse.SUPPRESS)
        parser_acr.add_argument('--common_passwords_file', help=argparse.SUPPRESS)
        parser_acr.add_argument('--include_patterns', help=argparse.SUPPRESS)
        parser_acr.add_argument('--include_patterns_file', help=argparse.SUPPRESS)
        parser_acr.add_argument('--exclude_patterns', help=argparse.SUPPRESS)
        parser_acr.add_argument('--exclude_patterns_file', help=argparse.SUPPRESS)
        parser_acr.add_argument('--mask_secret', action='store_true', help=argparse.SUPPRESS)
        parser_acr.add_argument('--no_code', action='store_true', help=argparse.SUPPRESS)
        parser_acr.add_argument('--sast', action='store_true', help=argparse.SUPPRESS)

        # Arguments required for Google Cloud Platform discovery
        parser_gcp = subparsers.add_parser ("gcp", help = "Discover Google Cloud Platform (GCP) instances")
        parser_gcp.add_argument('--enable_tracking_tags', action='store_true', help='Enable recording GCP specific information (like Project ID, etc.) as asset tags', required=False)

        # Arguments required for Google Cloud Registry container discovery 
        parser_gcr = subparsers.add_parser ("gcr", help = "Discover Google Cloud Registry (GCR) container images")
        parser_gcr.add_argument('--repository', help='The GCR image respository url which needs to be inspected.')
        parser_gcr.add_argument('--image', help='The fully qualified image name (with tag / digest) which needs to be inspected. If tag / digest is not given, latest will be determined and used.')
        parser_gcr.add_argument('--tmp_dir', help='Temporary directory. Defaults to /tmp', required=False)
        parser_gcr.add_argument('--containerid', help=argparse.SUPPRESS, required=False)
        parser_gcr.add_argument('--assetid', help=argparse.SUPPRESS, required=False)
        parser_gcr.add_argument('--assetname', help=argparse.SUPPRESS, required=False)
        parser_gcr.add_argument('--start_instance', action='store_true', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--repo', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--type', choices=repo.SUPPORTED_TYPES, help=argparse.SUPPRESS)
        parser_gcr.add_argument('--level', help=argparse.SUPPRESS, choices=['shallow','deep'], default='shallow')
        parser_gcr.add_argument('--secrets_scan', action='store_true', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--enable_entropy', action='store_true', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--regex_rules_file', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--check_common_passwords', action='store_true', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--common_passwords_file', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--include_patterns', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--include_patterns_file', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--exclude_patterns', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--exclude_patterns_file', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--mask_secret', action='store_true', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--no_code', action='store_true', help=argparse.SUPPRESS)
        parser_gcr.add_argument('--sast', action='store_true', help=argparse.SUPPRESS)

        # Arguments required for docker discovery 
        parser_docker = subparsers.add_parser ("docker", help = "Discover docker instances")
        parser_docker.add_argument('--image', help='The docker image (repo:tag) which needs to be inspected. If tag is not given, "latest" will be assumed.')
        parser_docker.add_argument('--containerid', help='The container ID of a running docker container which needs to be inspected.')
        parser_docker.add_argument('--assetid', help=argparse.SUPPRESS)
        parser_docker.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')
        parser_docker.add_argument('--tmp_dir', help='Temporary directory. Defaults to /tmp', default='/tmp')
        parser_docker.add_argument('--start_instance', action='store_true', help='If image inventory fails, try starting a container instance to inventory contents. Use with caution', required=False)
        parser_docker.add_argument('--repo', help=argparse.SUPPRESS)
        parser_docker.add_argument('--type', choices=repo.SUPPORTED_TYPES, help=argparse.SUPPRESS)
        parser_docker.add_argument('--level', help=argparse.SUPPRESS, choices=['shallow','deep'], default='shallow')
        parser_docker.add_argument('--secrets_scan', action='store_true', help=argparse.SUPPRESS)
        parser_docker.add_argument('--enable_entropy', action='store_true', help=argparse.SUPPRESS)
        parser_docker.add_argument('--regex_rules_file', help=argparse.SUPPRESS)
        parser_docker.add_argument('--check_common_passwords', action='store_true', help=argparse.SUPPRESS)
        parser_docker.add_argument('--common_passwords_file', help=argparse.SUPPRESS)
        parser_docker.add_argument('--include_patterns', help=argparse.SUPPRESS)
        parser_docker.add_argument('--include_patterns_file', help=argparse.SUPPRESS)
        parser_docker.add_argument('--exclude_patterns', help=argparse.SUPPRESS)
        parser_docker.add_argument('--exclude_patterns_file', help=argparse.SUPPRESS)
        parser_docker.add_argument('--mask_secret', action='store_true', help=argparse.SUPPRESS)
        parser_docker.add_argument('--no_code', action='store_true', help=argparse.SUPPRESS)
        parser_docker.add_argument('--sast', action='store_true', help=argparse.SUPPRESS)


        # Arguments required for Host discovery on Linux
        parser_linux = subparsers.add_parser ("host", help = "Discover linux host assets")
        parser_linux.add_argument('--remote_hosts_csv', help='CSV file containing details of remote hosts. CSV file column header [1st row] should be: hostname,userlogin,userpwd,privatekey,assetid,assetname. Note "hostname" column can contain hostname, IP address, CIDR range.')
        parser_linux.add_argument('--host_list', help='Same as the option: remote_hosts_csv. A file (currently in CSV format) containing details of remote hosts. CSV file column header [1st row] should be: hostname,userlogin,userpwd,privatekey,assetid,assetname. Note "hostname" column can contain hostname, IP address, CIDR range.')
        parser_linux.add_argument('--secure', action='store_true', help='Use this option to encrypt clear text passwords in the host list file')
        parser_linux.add_argument('--password', help='A password used to encrypt / decrypt login information from the host list file')
        parser_linux.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset')
        parser_linux.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')
        parser_linux.add_argument('--no_ssh_audit', action='store_true', help='Skip ssh audit')
        parser_linux.add_argument('--no_host_benchmark', action='store_true', help='Skip host benchmark audit')

        # Arguments required for nmap discovery
        parser_nmap = subparsers.add_parser ("nmap", help = "Discover assets using nmap")
        parser_nmap.add_argument('--hosts', help='A hostname, IP address or CIDR range', required=True)
        parser_nmap.add_argument('--no_ssh_audit', action='store_true', help='Skip ssh audit')

        # Arguments required for Repo discovery
        parser_repo = subparsers.add_parser ("repo", help = "Discover project repository as asset")
        parser_repo.add_argument('--repo', help='Local path or git repo url for project', required=True)
        parser_repo.add_argument('--type', choices=repo.SUPPORTED_TYPES, help='Type of open source component to scan for. Defaults to all supported types if not specified', required=False)
        parser_repo.add_argument('--level', help='Possible values {shallow, deep}. Shallow restricts discovery to 1st level dependencies only. Deep discovers dependencies at all levels. Defaults to shallow discovery if not specified', choices=['shallow','deep'], required=False, default='shallow')
        parser_repo.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset')
        parser_repo.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')
        # Switches related to secrets scan for repo
        parser_repo.add_argument('--secrets_scan', action='store_true', help='Perform a scan to look for secrets in the code')
        parser_repo.add_argument('--enable_entropy', action='store_true', help='Identify entropy based secrets')
        parser_repo.add_argument('--regex_rules_file', help='Path to JSON file specifying regex rules')
        parser_repo.add_argument('--check_common_passwords', action='store_true', help='Look for top common passwords.')
        parser_repo.add_argument('--common_passwords_file', help='Specify your own common passwords file. One password per line in file')
        parser_repo.add_argument('--include_patterns', help='Specify patterns which indicate files to be included in the secrets scan. Separate multiple patterns with comma.')
        parser_repo.add_argument('--include_patterns_file', help='Specify file containing include patterns which indicate files to be included in the secrets scan. One pattern per line in file.')
        parser_repo.add_argument('--exclude_patterns', help='Specify patterns which indicate files to be excluded in the secrets scan. Separate multiple patterns with comma.')
        parser_repo.add_argument('--exclude_patterns_file', help='Specify file containing exclude patterns which indicate files to be excluded in the secrets scan. One pattern per line in file.')
        parser_repo.add_argument('--mask_secret', action='store_true', help='Mask identified secret before storing for reference in ThreatWatch.')
        parser_repo.add_argument('--no_code', action='store_true', help='Disable storing code for reference in ThreatWatch.')
        parser_repo.add_argument('--sast', action='store_true', help='Perform static code analysis on your source code')

        # Arguments required for File-based discovery
        parser_file = subparsers.add_parser ("file", help = "Ingest asset inventory from file")
        parser_file.add_argument('--input', help='Absolute path to single input inventory file or a directory containing JSON or CSV files. Supported file formats are: CSV, JSON & PDF', required=True)
        parser_file.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset. Defaults to input filename if not specified. Applies only for PDF files.')
        parser_file.add_argument('--assetname', help='A name/label to be assigned to the discovered asset. Defaults to assetid is not specified. Applies only for PDF files.')
        parser_file.add_argument('--type', choices=['repo'], help='Type of asset. Defaults to repo if not specified. Applies only for PDF files.', required=False, default='repo')

        # Arguments required for ServiceNow discovery
        parser_snow = subparsers.add_parser ("servicenow", help = "Ingest inventory from ServiceNow CMDB")
        parser_snow.add_argument('--snow_user', help='User name of ServiceNow account', required=True)
        parser_snow.add_argument('--snow_user_pwd', help='User password of ServiceNow account', required=True)
        parser_snow.add_argument('--snow_instance', help='ServiceNow Instance name', required=True)
        parser_snow.add_argument('--enable_tracking_tags', action='store_true', help='Enable recording ServiceNow specific information (like ServiceNow instance name, etc.) as asset tags', required=False)

        # Arguments required for docker CIS benchmarks 
        parser_docker_cis = subparsers.add_parser ("docker_cis", help = "Run docker CIS benchmarks")
        parser_docker_cis.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset')
        parser_docker_cis.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')
        parser_docker_cis.add_argument('--docker_bench_home', help='Location of docker bench CLI', default='.')

        # Arguments required for AWS CIS benchmarks
        parser_aws_cis = subparsers.add_parser ("aws_cis", help = "Run AWS CIS benchmarks")
        parser_aws_cis.add_argument('--aws_access_key', help='AWS access key', required=True)
        parser_aws_cis.add_argument('--aws_secret_key', help='AWS secret key', required=True)
        parser_aws_cis.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset', required=True)
        parser_aws_cis.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')
        parser_aws_cis.add_argument('--prowler_home', help='Location of cloned prowler github repo. Defaults to /usr/share/prowler', default='/usr/share/prowler')

        # Arguments required for Azure CIS benchmarks
        parser_az_cis = subparsers.add_parser("azure_cis", help = "Run Azure CIS benchmarks")
        parser_az_cis.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset', required=True)
        parser_az_cis.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')

        # Arguments required for GCP CIS benchmarks
        parser_gcp_cis = subparsers.add_parser("gcp_cis", help = "Run Google Cloud Platform CIS benchmarks")
        parser_gcp_cis.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset', required=True)
        parser_gcp_cis.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')

        # Arguments required for ssl audit 
        parser_ssl_audit = subparsers.add_parser ("ssl_audit", help = "Run SSL audit tests against your web URLs. Requires [twigs_ssl_audit] package to be installed")
        parser_ssl_audit.add_argument('--url', help='HTTPS URL', required=True)
        parser_ssl_audit.add_argument('--args', help='Optional extra arguments')
        parser_ssl_audit.add_argument('--info', help='Report LOW / INFO level issues', action='store_true')
        parser_ssl_audit.add_argument('--assetid', help='A unique ID to be assigned to the discovered web URL asset', required=True)
        parser_ssl_audit.add_argument('--assetname', help='Optional name/label to be assigned to the web URL asset')

        # Arguments required for web-app discovery and testing
        parser_webapp = subparsers.add_parser ("dast", help = "Discover and test web application using a DAST plugin")
        parser_webapp.add_argument('--url', help='Web application URL', required=True)
        parser_webapp.add_argument('--plugin', choices=['arachni', 'skipfish'], help='DAST plugin to be used. Default is arachni. Requires the plugin to be installed separately.', default='arachni')
        parser_webapp.add_argument('--pluginpath', help='Path where the DAST plugin is installed to be used. Default is /usr/bin.', default='/usr/bin')
        parser_webapp.add_argument('--args', help='Optional extra arguments to be passed to the plugin')
        parser_webapp.add_argument('--assetid', help='A unique ID to be assigned to the discovered webapp asset', required=True)
        parser_webapp.add_argument('--assetname', help='Optional name/label to be assigned to the webapp asset')


        args = parser.parse_args()

        logging_level = logging.WARNING
        if args.verbosity >= 1:
            logging_level = logging.INFO
        if args.verbosity >= 2:
            logging_level = logging.DEBUG
        if args.quiet:
            logging_level = logging.ERROR
        # Setup the logger
        logging.basicConfig(filename=logfilename, level=logging_level, filemode='w', format='%(asctime)s %(levelname)-8s %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p')
        console = logging.StreamHandler()
        console.setLevel(logging_level)
        console.setFormatter(logging.Formatter('%(levelname)-8s %(message)s'))
        logging.getLogger('').addHandler(console)

        # In insecure mode, we want to set verify=False for requests
        if args.insecure:
            utils.set_requests_verify(False)

        logging.info('Started new run')
        logging.debug('Arguments: %s', str(args))

        if args.mode == "login":
            login_user(args)
        elif args.mode == "logout":
            logout_user(args)

        logged_in_user_dict = { }

        if args.handle is None:
            logged_in_user_dict = get_logged_in_user_details()
            temp = logged_in_user_dict.get('handle')
            if temp is None:
                temp = os.environ.get('TW_HANDLE')
                if temp is None:
                    logging.error('Error: Missing "--handle" argument, user not logged in twigs and "TW_HANDLE" environment variable is not set as well')
                    sys.exit(1)
                logging.info('Using handle specified in "TW_HANDLE" environment variable')
            else:
                logging.info('Using handle of logged in user')
            args.handle = temp

        if args.token is None:
            temp = logged_in_user_dict.get('token')
            if temp is None:
                temp = os.environ.get('TW_TOKEN')
                if temp is not None:
                    logging.info('Using token specified in "TW_TOKEN" environment variable')
            else:
                logging.info('Using token of logged in user')
            args.token = temp

        if args.token is None and args.apply_policy is not None:
            logging.error('Error: Policy cannot be applied since "--token" argument is missing and "TW_TOKEN" environment variable is not set as well!')
            sys.exit(1)

        if args.instance is None:
            temp = logged_in_user_dict.get('instance')
            if temp is None:
                temp = os.environ.get('TW_INSTANCE')
                if temp is not None:
                    logging.info('Using instance specified in "TW_INSTANCE" environment variable')
                    args.instance = temp
                elif args.token is not None:
                    # missing instance but token is specified
                    logging.error('Error: Missing "--instance" argument and "TW_INSTANCE" environment variable is not set as well')
                    sys.exit(1)
            else:
                logging.info('Using instance of logged in user')
                args.instance = temp

#    if args.purge_assets == True and args.email_report == False:
#        logging.error('Purge assets option (--purge_assets) is used with Email report (--email_report)')
#        sys.exit(1)

        if (args.token is None or len(args.token) == 0) and args.out is None:
            logging.error('[token] argument is not specified and [out] argument is not specified. Unable to share discovered assets.')
            sys.exit(1)

        if args.schedule is not None and sys.platform != 'win32':
            from crontab import CronSlices
            # validate schedule
            if CronSlices.is_valid(args.schedule) == False:
                logging.error("Error: Invalid cron schedule [%s] specified!" % args.schedule)
                sys.exit(1)

        assets = []
        sub_pkg_list = ['ssl_audit']
        if args.mode in sub_pkg_list:
            assets = sub_pkg_get_inventory(args)
        elif args.mode == 'aws':
            assets = aws.get_inventory(args)
        elif args.mode == 'azure':
            assets = azure.get_inventory(args)
        elif args.mode == 'acr':
            assets = acr.get_inventory(args)
        elif args.mode == 'ecr':
            assets = ecr.get_inventory(args)
        elif args.mode == 'gcp':
            assets = gcp.get_inventory(args)
        elif args.mode == 'gcr':
            assets = gcr.get_inventory(args)
        elif args.mode == 'servicenow':
            assets = servicenow.get_inventory(args)
        elif args.mode == 'repo':
            assets = repo.get_inventory(args)
        elif args.mode == 'host':
            assets = linux.get_inventory(args)
        elif args.mode == 'nmap':
            assets = fingerprint.get_inventory(args)
        elif args.mode == 'docker':
            assets = docker.get_inventory(args)
        elif args.mode == 'file':
            assets = inv_file.get_inventory(args)
        elif args.mode == 'dast':
            assets = dast.get_inventory(args)
        elif args.mode == 'docker_cis':
            assets = docker_cis.get_inventory(args)
        elif args.mode == 'aws_cis':
            assets = aws_cis.get_inventory(args)
        elif args.mode == 'azure_cis':
            assets = azure_cis.get_inventory(args)
        elif args.mode == 'gcp_cis':
            assets = gcp_cis.get_inventory(args)
        elif args.mode == 'ssl_audit':
            assets = ssl_audit.get_inventory(args)

        exit_code = None
        if args.mode != 'host' or args.secure == False:
            if assets is None or len(assets) == 0:
                logging.info("No assets found!")
            else:
                """
                if args.asset_criticality is not None:
                    add_asset_criticiality_tag(assets, args.asset_criticality)
                """

                if args.tag_critical:
                    add_asset_criticality_tag(assets, '5')

                if args.tag:
                    add_asset_tags(assets, args.tag)

                if args.out is not None:
                    export_assets_to_file(assets, args.out)

                if args.token is not None and len(args.token) > 0:
                    asset_id_list, scan_asset_id_list = push_assets_to_TW(assets, args)

                pj_json = None
                if args.apply_policy is not None:
                    policy_job_name = policy_lib.apply_policy(args.apply_policy, asset_id_list, args)
                    while True:
                        time.sleep(60)
                        status, pj_json = policy_lib.is_policy_job_done(policy_job_name, args)
                        if status:
                            exit_code = policy_lib.process_policy_job_actions(pj_json)
                            break

                if args.token is not None and len(args.token) > 0:
                    run_scan(scan_asset_id_list, pj_json, args)
            
                if args.schedule is not None and sys.platform != 'win32':
                    from crontab import CronTab
                    # create/update cron job
                    cron_cmd = sys.argv[0]
                    if "--handle" not in sys.argv:
                        cron_cmd = cron_cmd + " " + "--handle " + '"' + args.handle + '"'
                    if "--token" not in sys.argv:
                        cron_cmd = cron_cmd + " " + "--token " + '"' + args.token + '"'
                    if "--instance" not in sys.argv:
                        cron_cmd = cron_cmd + " " + "--instance " + '"' + args.instance + '"'
                    skip_next = False
                    for index in range(1, len(sys.argv)):
                        if sys.argv[index] == "--schedule":
                            skip_next = True
                            continue
                        if skip_next:
                            skip_next = False
                            continue
                        if sys.argv[index].startswith('-'):
                            cron_cmd = cron_cmd + " " + sys.argv[index]
                        else:
                            cron_cmd = cron_cmd + " " + '"' + sys.argv[index] + '"'
                    cron_comment = "TWIGS_" + args.mode
                    with CronTab(user=True) as user_cron:
                        # Find any existing jobs and remove those
                        ejobs = user_cron.find_comment(cron_comment)
                        for ejob in ejobs:
                            user_cron.remove(ejob)
                        njob = user_cron.new(command=cron_cmd, comment=cron_comment)
                        njob.setall(args.schedule)
                        logging.info("Added to crontab with comment [%s]", cron_comment)

        logging.info('Run completed')
        if exit_code is not None:
            logging.info("Exiting with code [%s] based on policy evaluation", exit_code)
            sys.exit(exit_code)

    except Exception as e:
        logging.error("Something went wrong with this twigs run")
        st = traceback.format_exc()
        logging.error("Exception trace details: %s", st)
        sys.exit(1)

if __name__ == '__main__':
    main()



import sys
import datetime
import logging
from . import gcp_cis_utils as gcp_cis_utils

def check1_1():
    # 1.1 Ensure that corporate login credentials are used (Scored)

    logging.info("1.1 Ensure that corporate login credentials are used (Scored)")
    # Get all projects
    projects = gcp_cis_utils.get_all_projects()

    # List all organizations
    orgs = gcp_cis_utils.get_all_organizations()

    # List all folders
    folders = gcp_cis_utils.get_all_folders()

    # List accounts for each project
    details = []
    iam_policies_by_project = gcp_cis_utils.get_iam_policies_by_projects()
    for p in iam_policies_by_project.keys():
        out_json = iam_policies_by_project[p]
        for entry in out_json['bindings']:
            members = entry.get('members')
            for m in members:
                if m.endswith('gmail.com'):
                    details.append("Personal email [%s] is used for role [%s] in IAM policies for project [%s]" % (m.split(':')[1], entry['role'], p))
    
    # List accounts for each organization
    for o in orgs:
        out_json = gcp_cis_utils.run_gcloud_cmd("organizations get-iam-policy %s" % (o))
        for entry in out_json['bindings']:
            members = entry.get('members')
            for m in members:
                if m.endswith('gmail.com'):
                    details.append("Personal email [%s] is used for role [%s] in IAM policies for project [%s]" % (m.split(':')[1], entry['role'], o))

    # List accounts for each folder
    for f in folders:
        out_json = gcp_cis_utils.run_gcloud_cmd("resource-manager folders get-iam-policy " % f)
        for entry in out_json['bindings']:
            members = entry.get('members')
            for m in members:
                if m.endswith('gmail.com'):
                    details.append("Personal email [%s] is used for role [%s] in IAM policies for project [%s]" % (m.split(':')[1], entry['role'], f))

    # record violation
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-1.1', '1.1 [Level 1] Ensure that corporate login credentials are used (Scored)', "\n".join(details), '4', '', '')
    return None

def check1_2():
    # 1.2 Ensure that multi-factor authentication is enabled for all non-service accounts (Not Scored)
    logging.info("Not supported - 1.2 Ensure that multi-factor authentication is enabled for all non-service accounts (Not Scored)")
    return None

def check1_3():
    # 1.3 Ensure that Security Key Enforcement is enabled for all admin accounts (Not Scored)

    logging.info("Not supported - 1.3 Ensure that Security Key Enforcement is enabled for all admin accounts (Not Scored)")
    return None

def check1_4():

    # 1.4 Ensure that there are only GCP-managed service account keys for each service account (Scored)

    logging.info("1.4 Ensure that there are only GCP-managed service account keys for each service account (Scored)")
    details = []
    projects = gcp_cis_utils.get_all_projects()
    for p in projects:
        out_json = gcp_cis_utils.run_gcloud_cmd("iam service-accounts list --project=%s" % p)
        for entry in out_json:
            sa_email = entry['email']
            if sa_email.endswith('.iam.gserviceaccount.com'):
                out_json_2 = gcp_cis_utils.run_gcloud_cmd("iam service-accounts keys list --iam-account=%s --managed-by=user" % sa_email)
                for entry2 in out_json_2:
                    details.append("User managed service account [%s] has user-managed key" % sa_email)
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-1.4', '1.4 [Level 1] Ensure that there are only GCP-managed service account keys for each service account (Scored)', "\n".join(details), '4', '', '')
    return None

def check1_5():
    # 1.5 Ensure that Service Account has no Admin privileges (Scored)

    logging.info("1.5 Ensure that Service Account has no Admin privileges (Scored)")
    details = []
    iam_policies_by_project = gcp_cis_utils.get_iam_policies_by_projects()
    for p in iam_policies_by_project.keys():
        out_json = iam_policies_by_project[p]
        for entry in out_json['bindings']:
            if 'admin' in entry['role'].lower() or entry['role'] in ["roles/editor", "roles/owner"]:
                for m in entry['members']:
                    if m.endswith('.iam.gserviceaccount.com'):
                        details.append("User created and managed service account [%s] has role [%s] in project [%s]" % (m, entry['role'], p))
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-1.5', '1.5 [Level 1] Ensure that Service Account has no Admin privileges (Scored)', "\n".join(details), '4', '', '')
    return None

def check1_6():
    # 1.6 Ensure that IAM users are not assigned the Service Account User or Service Account Token Creator roles at project level (Scored)

    logging.info("1.6 Ensure that IAM users are not assigned the Service Account User or Service Account Token Creator roles at project level (Scored)")
    details = []
    iam_policies_by_project = gcp_cis_utils.get_iam_policies_by_projects()
    for p in iam_policies_by_project.keys():
        out_json = iam_policies_by_project[p]
        for entry in out_json['bindings']:
            if entry['role'] in ["roles/iam.serviceAccountUser", "roles/iam.serviceAccountTokenCreator"]:
                for m in entry['members']:
                    if m.startswith('user:'):
                        details.append("IAM user [%s] is assigned role [%s] in project [%s]" % (m.split(':')[1], entry['role'], p))
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-1.6', '1.6 [Level 1] Ensure that IAM users are not assigned the Service Account User or Service Account Token Creator roles at project level (Scored)', "\n".join(details), '4', '', '')
    return None

def check1_7():
    # 1.7 Ensure user-managed/external keys for service accounts are rotated every 90 days or less (Scored)

    logging.info("1.7 Ensure user-managed/external keys for service accounts are rotated every 90 days or less (Scored)")
    details = []
    last90days = datetime.datetime.now() - datetime.timedelta(days=90)
    projects = gcp_cis_utils.get_all_projects()
    for p in projects:
        out_json = gcp_cis_utils.run_gcloud_cmd("iam service-accounts list --project=%s" % p)
        for entry in out_json:
            out_json_2 = gcp_cis_utils.run_gcloud_cmd("iam service-accounts keys list --iam-account %s --managed-by=user" % entry['email'])
            for entry_2 in out_json_2:
                entry_2_vat = entry_2['validAfterTime']
                if '.' in entry_2_vat:
                    entry_2_vat = entry_2_vat.split('.')[0] + 'Z'
                vat = datetime.datetime.strptime(entry_2_vat, "%Y-%m-%dT%H:%M:%SZ")
                if vat > last90days:
                    details.append("User managed service account [%s] has not rotated key in last 90 days. Key has been valid since %s" % (entry['email'], entry_2['validAfterTime']))
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-1.7', '1.7 [Level 1] Ensure user-managed/external keys for service accounts are rotated every 90 days or less (Scored)', "\n".join(details), '4', '', '')
    return None

def check1_8():
    # 1.8 Ensure that Separation of duties is enforced while assigning service account related roles to users (Not Scored)

    logging.info("1.8 Ensure that Separation of duties is enforced while assigning service account related roles to users (Not Scored)")
    details = []
    iam_policies_by_projects = gcp_cis_utils.get_iam_policies_by_projects()
    for p in iam_policies_by_projects.keys():
        out_json = iam_policies_by_projects[p]
        saa_role_users = set()
        sau_role_users = set()
        for entry in out_json['bindings']:
            gcp_cis_utils.add_members_with_role(entry, "roles/iam.serviceAccountAdmin", saa_role_users)
            gcp_cis_utils.add_members_with_role(entry, "roles/iam.serviceAccountUser", sau_role_users)
        intersection = saa_role_users.intersection(sau_role_users)
        if len(intersection) > 0:
            details.append("Separation of duties is not enforced for IAM users [%s] for roles (Service Account Admin & Service Account User) in project [%s]" % (",".join(list(intersection)), p))
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-1.8', '1.8 [Level 2] Ensure that Separation of duties is enforced while assigning service account related roles to users (Not Scored)', "\n".join(details), '5', '', '')
    return None

def check1_9():
    # 1.9 Ensure that Cloud KMS cryptokeys are not anonymously or publicly accessible (Scored)

    logging.info("1.9 Ensure that Cloud KMS cryptokeys are not anonymously or publicly accessible (Scored)")
    details = []
    projects = gcp_cis_utils.get_cloud_kms_enabled_projects()
    for p in projects:
        out_json = gcp_cis_utils.run_gcloud_cmd("kms keyrings list --location=global --project=%s" % p)
        for entry in out_json:
            out_json_2 = gcp_cis_utils.run_gcloud_cmd("kms keys list --keyring=%s --location=global" % entry['name'])
            for entry_2 in out_json_2:
                out_json_3 = gcp_cis_utils.run_gcloud_cmd("kms keys get-iam-policy %s --keyring=%s --location=global" % (entry_2['name'], entry['name']))
                bindings = out_json_3.get('bindings')
                if bindings is None:
                    continue
                for entry_3 in bindings:
                    if "allUsers" in entry_3['members'] or "allAuthenticatedUsers" in entry_3['members']:
                        details.append("Cloud KMS CryptoKey [%s] is anonymously or publically accessible in project [%s]" % (entry_2['name'], p))
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-1.9', '1.9 [Level 1] Ensure that Cloud KMS cryptokeys are not anonymously or publicly accessible (Scored)', "\n".join(details), '4', '', '')
    return None

def check1_10():
    # 1.10 Ensure KMS encryption keys are rotated within a period of 90 days (Scored)

    logging.info("1.10 Ensure KMS encryption keys are rotated within a period of 90 days (Scored)")
    details = []
    next90days = datetime.datetime.now() + datetime.timedelta(days=90)
    rpd = { 'm': (129600, 'minutes'), 's': (7776000, 'seconds'), 'h': (2160, 'hours'), 'd': (90, 'days') }
    projects = gcp_cis_utils.get_cloud_kms_enabled_projects()
    for p in projects:
        out_json = gcp_cis_utils.run_gcloud_cmd("kms keyrings list --location=global --project=%s" % p)
        for entry in out_json:
            out_json_2 = gcp_cis_utils.run_gcloud_cmd("kms keys list --keyring=%s --location=global" % entry['name'])
            for entry_2 in out_json_2:
                rotationPeriod = entry_2.get('rotationPeriod')
                nextRotationTime = entry_2.get('nextRotationTime')
                if rotationPeriod is not None:
                    rpd_entry = rpd[rotationPeriod[-1]]
                    if int(rotationPeriod[:-1]) > rpd_entry[0]:
                        details.append("Rotation period for key [%s] in project [%s] exceeeds 90 days. Currently set to rotate every [%s] %s" % (entry_2['name'], p, rotationPeriod[:-1], rpd_entry[1]))
                if nextRotationTime is not None:
                    if '.' in nextRotationTime:
                        nextRotationTime = nextRotationTime.split('.')[0] + 'Z'
                    nrt = datetime.datetime.strptime(nextRotationTime, "%Y-%m-%dT%H:%M:%SZ")
                    if nrt > next90days:
                        details.append("Next rotation time for key [%s] in project [%s] exceeds 90 days from now. Currently set to rotate next at [%s]" % (entry_2['name'], p, nextRotationTime))
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-1.10', '1.10 [Level 1] Ensure KMS encryption keys are rotated within a period of 90 days (Scored)', "\n".join(details), '4', '', '')
    return None

def check1_11():
    # 1.11 Ensure that Separation of duties is enforced while assigning KMS related roles to users (Scored)

    logging.info("1.11 Ensure that Separation of duties is enforced while assigning KMS related roles to users (Scored)")
    details = []
    iam_policies_by_projects = gcp_cis_utils.get_iam_policies_by_projects()
    for p in iam_policies_by_projects.keys():
        out_json = iam_policies_by_projects[p]
        bindings = out_json.get('bindings')
        if bindings is None:
            continue
        ckms_admin_set = set()
        ckms_cke_set = set()
        ckms_ckd_set = set()
        ckms_cked_set = set()
        for entry in bindings:
            gcp_cis_utils.add_members_with_role(entry, "roles/cloudkms.admin", ckms_admin_set)
            gcp_cis_utils.add_members_with_role(entry, "roles/cloudkms.cryptoKeyEncrypter", ckms_cke_set)
            gcp_cis_utils.add_members_with_role(entry, "roles/cloudkms.cryptoKeyDecrypter", ckms_ckd_set)
            gcp_cis_utils.add_members_with_role(entry, "roles/cloudkms.cryptoKeyEncrypterDecrypter", ckms_cked_set)
        intersection = ckms_admin_set.intersection(ckms_cke_set)
        if len(intersection) > 0:
            details.append("Separation of duties is not enforced for IAM users [%s] for roles (Cloud KMS Admin & Cloud KMS CryptoKey Encrypter) in project [%s]" % (",".join(list(intersection)), p))
        intersection = ckms_admin_set.intersection(ckms_ckd_set)
        if len(intersection) > 0:
            details.append("Separation of duties is not enforced for IAM users [%s] for roles (Cloud KMS Admin & Cloud KMS CryptoKey Decrypter) in project [%s]" % (",".join(list(intersection)), p))
        intersection = ckms_admin_set.intersection(ckms_cked_set)
        if len(intersection) > 0:
            details.append("Separation of duties is not enforced for IAM users [%s] for roles (Cloud KMS Admin & Cloud KMS CryptoKey Encrypter/Decrypter) in project [%s]" % (",".join(list(intersection)), p))
    if len(details) > 0:
        return gcp_cis_utils.create_issue('cis-gcp-bench-check-1.11', '1.11 [Level 2] Ensure that Separation of duties is enforced while assigning KMS related roles to users (Scored)', "\n".join(details), '5', '', '')
    return None

def check1_12():
    # 1.12 Ensure API keys are not created for a project (Not Scored)

    logging.info("Not supported - 1.12 Ensure API keys are not created for a project (Not Scored)")
    return None

def check1_13():
    # 1.13 Ensure API keys are restricted to use by only specified Hosts and Apps (Not Scored)

    logging.info("Not supported - 1.13 Ensure API keys are restricted to use by only specified Hosts and Apps (Not Scored)")
    return None

def check1_14():
    # 1.14 Ensure API keys are restricted to only APIs that application needs access (Not Scored)

    logging.info("Not supported - 1.14 Ensure API keys are restricted to only APIs that application needs access (Not Scored)")
    return None

def check1_15():
    # 1.15 Ensure API keys are rotated every 90 days (Not Scored)

    logging.info("Not supported - 1.15 Ensure API keys are rotated every 90 days (Not Scored)")
    return None

def run_checks():
    config_issues = []
    gcp_cis_utils.append_issue(config_issues, check1_1())
    gcp_cis_utils.append_issue(config_issues, check1_2())
    gcp_cis_utils.append_issue(config_issues, check1_3())
    gcp_cis_utils.append_issue(config_issues, check1_4())
    gcp_cis_utils.append_issue(config_issues, check1_5())
    gcp_cis_utils.append_issue(config_issues, check1_6())
    gcp_cis_utils.append_issue(config_issues, check1_7())
    gcp_cis_utils.append_issue(config_issues, check1_8())
    gcp_cis_utils.append_issue(config_issues, check1_9())
    gcp_cis_utils.append_issue(config_issues, check1_10())
    gcp_cis_utils.append_issue(config_issues, check1_11())
    gcp_cis_utils.append_issue(config_issues, check1_12())
    gcp_cis_utils.append_issue(config_issues, check1_13())
    gcp_cis_utils.append_issue(config_issues, check1_14())
    gcp_cis_utils.append_issue(config_issues, check1_15())
    return config_issues



import os
import sys
import json
import re
import math
import mmap
from . import utils as lib_utils
from . import code_secrets_defaults as cs_defaults

BASE64_CHARACTERS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/="
HEX_CHARACTERS = "1234567890abcdefABCDEF"
regex_rules = { }
common_pwds = [ ]
textchars = bytearray({7,8,9,10,12,13,27} | set(range(0x20, 0x100)) - {0x7f})
is_binary_string = lambda in_bytes: bool(in_bytes.translate(None, textchars))

def shannon_entropy(data, iterator):
    if not data:
        return 0
    entropy = 0
    for x in iterator:
        p_x = float(data.count(x))/len(data)
        if p_x > 0:
            entropy += - p_x*math.log(p_x, 2)
    return entropy

def extract_strings(word, charset, threshold=20):
    count = 0
    letters = ""
    strings = []
    for char in word:
        if char in charset:
            letters += char
            count += 1
        else:
            if count > threshold:
                strings.append(letters)
            letters = ""
            count = 0
    if count > threshold:
        strings.append(letters)
    return strings

def hide_secrets(lines):
    ret_lines = []
    for line_content in lines.split('\n'):
        for word in line_content.split():
            b64_strings = extract_strings(word, BASE64_CHARACTERS)
            hex_strings = extract_strings(word, HEX_CHARACTERS)
            for string in b64_strings:
                base64_entropy = shannon_entropy(string, BASE64_CHARACTERS)
                if base64_entropy > 4.5:
                    line_content = line_content.replace(string, "*" * len(string))
            for string in hex_strings:
                hex_entropy = shannon_entropy(string, HEX_CHARACTERS)
                if hex_entropy > 3:
                    line_content = line_content.replace(string, "*" * len(string))
        for key in regex_rules:
            matched = regex_rules[key].search(line_content)
            if matched:
                string = matched.group()
                line_content = line_content.replace(string, "*" * len(string))
        for cp in common_pwds:
            matched = cp.search(line_content)
            if matched:
                string = matched.group()[1:-1] # remove the qoutes
                line_content = line_content.replace(string, "*" * len(string))
        ret_lines.append(line_content)
    return "\n".join(ret_lines)

def truncate_code_snippet(secret_record):
    max_line_length = 1000 # Column data type allows 65535 characters and there are 5 lines
    truncated_indicator = '...truncated...'
    if len(secret_record['line_content']) > max_line_length:
        truncated = secret_record['line_content'][:max_line_length]
        secret_record['line_content'] = truncated + truncated_indicator
    truncated = ''
    for line in secret_record['before_content'].split('\n'):
        if len(truncated) > 0:
            truncated = truncated + '\n'
        if len(line) > max_line_length:
            truncated = truncated + line[:max_line_length] + truncated_indicator
        else:
            truncated = truncated + line
    secret_record['before_content'] = truncated
    truncated = ''
    for line in secret_record['after_content'].split('\n'):
        if len(truncated) > 0:
            truncated = truncated + '\n'
        if len(line) > max_line_length:
            truncated = truncated + line[:max_line_length] + truncated_indicator
        else:
            truncated = truncated + line
    secret_record['after_content'] = truncated

def create_secret_record(filename, lines, line_no, record_type, line_content, secret, args):
    to_mask = args.mask_secret
    secret_record = { }
    secret_record['filename'] = filename
    secret_record['line_no'] = line_no + 1
    secret_record['discovered_using'] = record_type.split(':')[0]
    if secret_record['discovered_using'] == 'REGEX':
        secret_record['regex'] = record_type[len(secret_record['discovered_using'])+1:]
    if line_no >= 2:
        before_content = lib_utils.ascii_string(lines[line_no-2]) + '\n' + lib_utils.ascii_string(lines[line_no-1])
    elif line_no == 1:
        before_content = lib_utils.ascii_string(lines[line_no-1])
    else:
        before_content = ''
    if line_no < len(lines) - 2:
        after_content = lib_utils.ascii_string(lines[line_no+1]) + '\n' + lib_utils.ascii_string(lines[line_no+2])
    elif line_no == len(lines) - 2:
        after_content = lib_utils.ascii_string(lines[line_no+1])
    else:
        after_content = ''
    if args.no_code:
        secret_record['line_content'] = ''
        secret_record['before_content'] = ''
        secret_record['after_content'] = ''
    else:
        secret_length = len(secret)
        line_content = lib_utils.ascii_string(line_content)
        secret_record['column_start'] = line_content.find(secret)
        if secret_record['column_start'] != -1:
            secret_record['column_end'] = secret_record['column_start'] + secret_length - 1
        else:
            secret_record['column_end'] = -1
        if to_mask:
            line_content = line_content.replace(secret, "*" * secret_length)
        secret_record['line_content'] = line_content
        secret_record['before_content'] = hide_secrets(before_content) if to_mask else before_content
        secret_record['after_content'] = hide_secrets(after_content) if to_mask else after_content
        truncate_code_snippet(secret_record)
    return secret_record

def check_entropy(this_file, lines, line, line_no, secret_records, args):
    for word in line.split():
        b64_strings = extract_strings(word, BASE64_CHARACTERS)
        hex_strings = extract_strings(word, HEX_CHARACTERS)
        for string in b64_strings:
            base64_entropy = shannon_entropy(string, BASE64_CHARACTERS)
            if base64_entropy > 4.5:
                secret = string
                secret_records.append(create_secret_record(this_file, lines, line_no, "ENTROPY_BASE64", line, secret, args))
                break
        for string in hex_strings:
            hex_entropy = shannon_entropy(string, HEX_CHARACTERS)
            if hex_entropy > 3:
                secret = string
                secret_records.append(create_secret_record(this_file, lines, line_no, "ENTROPY_HEX", line, secret, args))
                break
    return

def check_common_passwords(this_file, lines, line, line_no, secret_records, args):
    for cp in common_pwds:
        matched = cp.search(line)
        if matched:
            secret = matched.group()[1:-1] # remove the qoutes
            secret_records.append(create_secret_record(this_file, lines, line_no, "COMMON_PASSWORD", line, secret, args))
            break

def check_regex_rules(this_file, lines, line, line_no, secret_records, args):
    for key in regex_rules:
        matched = regex_rules[key].search(line)
        if matched:
            secret = matched.group()
            secret_records.append(create_secret_record(this_file, lines, line_no, "REGEX:"+key, line, secret, args))
            break

def scan_file_for_secrets(args, base_path, this_file, regex_rules):
    secret_records = []
    with open(this_file, 'r') as fd:
        if sys.platform == 'win32':
            mm_file = mmap.mmap(fd.fileno(), 0, access=mmap.ACCESS_READ)
        else:
            mm_file = mmap.mmap(fd.fileno(), 0, prot=mmap.PROT_READ)
        lines = mm_file.read(-1)
        lines = lines.decode(args.encoding)
        lines = lines.split('\n')
        line_no = 0
        stripped_file_path = this_file[len(base_path)+1:]
        for line in lines:
            if args.enable_entropy:
                check_entropy(stripped_file_path, lines, line, line_no, secret_records, args)
            check_regex_rules(stripped_file_path, lines, line, line_no, secret_records, args)
            if args.check_common_passwords:
                check_common_passwords(stripped_file_path, lines, line, line_no, secret_records, args)
            line_no = line_no + 1
    return secret_records

def read_patterns(patterns, patterns_file, msg):
    temp_patterns = []
    if patterns:
        temp_patterns = patterns.split(',')
    if patterns_file:
        if os.path.isfile(patterns_file) == False:
            logging.error("Error unable to read patterns file [%s]", patterns_file)
            sys.exit(1)
        with open(patterns_file, 'r') as fd:
            data = fd.read()
            temp_patterns = data.split('\n')
    if patterns and patterns_file:
        logging.info(msg)

    ret_patterns = []
    for pattern in temp_patterns:
        ret_patterns.append(re.compile(pattern))
    return ret_patterns

def meets_pattern(this_file, patterns):
    for pattern in patterns:
        if pattern.search(this_file):
            return True
    return False

def scan_for_secrets(args, local_path, base_path):
    local_path = os.path.abspath(local_path)
    all_files = lib_utils.find_files(local_path, '')

    include_patterns = read_patterns(args.include_patterns, args.include_patterns_file, "Both include_patterns and include_patterns_file options are specified. Only include_patterns_file will be considered")

    exclude_patterns = []
    # Load default_exclude_patterns
    for dp in cs_defaults.default_exclude_patterns:
        exclude_patterns.append(re.compile(dp))
    user_exclude_patterns = read_patterns(args.exclude_patterns, args.exclude_patterns_file, "Both exclude_patterns and exclude_patterns_file options are specified. Only exclude_patterns_file will be considered")
    exclude_patterns.extend(user_exclude_patterns)

    final_files = []
    for this_file in all_files:
        if len(exclude_patterns) == 0 or (len(exclude_patterns) > 0 and meets_pattern(this_file, exclude_patterns) == False):
            if len(include_patterns) == 0 or (len(include_patterns) > 0 and meets_pattern(this_file, include_patterns) == True):
                final_files.append(this_file)

    global regex_rules
    if args.regex_rules_file:
        with open(args.regex_rules_file, 'r') as fd:
            regex_rules = json.load(fd)
    else:
        regex_rules = cs_defaults.default_regex_rules
    for key in regex_rules:
        regex_rules[key] = re.compile(regex_rules[key]) # store the precompiled regex

    global common_pwds
    if args.check_common_passwords:
        if args.common_passwords_file:
            if os.path.isfile(args.common_passwords_file) == False:
                logging.error("Error unable to read common passwords file [%s]", args.common_passwords_file)
                sys.exit(1)
            with open(args.common_passwords_file, 'r') as fd:
                buf = fd.read()
            common_passwords_list = buf.split('\n')
        else:
            common_passwords_list = cs_defaults.common_passwords
        for cp in common_passwords_list:
            cp = cp.strip()
            if len(cp) > 0: # Safety check
                common_pwds.append(re.compile("[^a-zA-Z0-9]"+cp+"[^a-zA-Z0-9]"))

    secret_records = []
    for this_file in final_files:
        if os.path.islink(this_file) == False and os.stat(this_file).st_size > 0 and is_binary_string(open(this_file, 'rb').read(1024)) == False:
            secret_records.extend(scan_file_for_secrets(args, base_path, this_file, regex_rules))

    return secret_records

